@@ -603,6 +603,10 @@ init_post (void)
     gboolean changed = FALSE;
     GList *l;
 
+#ifdef HAVE_FORK
+    pid_t pid;
+#endif
+
     for (l = plugin_paths; l != NULL; l = l->next) {
       GST_INFO ("Scanning plugin path: \"%s\"", (gchar *) l->data);
       /* CHECKME: add changed |= here as well? */
@@ -612,10 +616,6 @@ init_post (void)
     g_list_free (plugin_paths);
     plugin_paths = NULL;
 
-#ifdef HAVE_FORK
-    pid_t pid;
-#endif
-
     default_registry = gst_registry_get_default ();
     registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
     if (registry_file == NULL) {
@@ -695,7 +695,9 @@ init_post (void)
       _gst_registry_remove_cache_plugins (default_registry);
 
 #ifdef HAVE_FORK
-      exit (0);
+      /* need to use _exit, so that any exit handlers registered don't
+       * bring down the main program */
+      _exit (0);
     } else {
       /* parent */
       int status;
