@@ -141,13 +141,13 @@
  * </variablelist>
  *
  * A #GstBin will by default forward any event sent to it to all sink elements.
- * If all the sinks return TRUE, the bin will also return TRUE. If no sinks are
- * in the bin, the event handler will return TRUE.
+ * If all the sinks return TRUE, the bin will also return TRUE, else FALSE is
+ * returned. If no sinks are in the bin, the event handler will return TRUE.
  *
  * </para>
  * </refsect2>
  *
- * Last reviewed on 2006-03-12 (0.10.5)
+ * Last reviewed on 2006-04-28 (0.10.6)
  */
 
 #include "gst_private.h"
@@ -167,7 +167,8 @@
 GST_DEBUG_CATEGORY_STATIC (bin_debug);
 #define GST_CAT_DEFAULT bin_debug
 
-static GstElementDetails gst_bin_details = GST_ELEMENT_DETAILS ("Generic bin",
+static const GstElementDetails gst_bin_details =
+GST_ELEMENT_DETAILS ("Generic bin",
     "Generic/Bin",
     "Simple container object",
     "Erik Walthinsen <omega@cse.ogi.edu>," "Wim Taymans <wim@fluendo.com>");
@@ -1895,7 +1896,9 @@ done:
 
 /*
  * This function is a utility event handler for seek events.
- * It will send the event to all sinks.
+ * It will send the event to all sinks or sources depending on the
+ * event-direction.
+ *
  * Applications are free to override this behaviour and
  * implement their own seek handler, but this will work for
  * pretty much all cases in practice.
@@ -2065,9 +2068,9 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
       gst_message_unref (message);
 
       klass = GST_BIN_GET_CLASS (bin);
-      gst_object_ref (bin);
       if (!bin->polling) {
         GST_DEBUG_OBJECT (bin, "pushing recalc on thread pool");
+        gst_object_ref (bin);
         g_thread_pool_push (klass->pool, bin, NULL);
       } else {
         GST_DEBUG_OBJECT (bin,
