@@ -48,7 +48,9 @@
  * bus to handle them.
  * Alternatively the application can register an asynchronous bus function
  * using gst_bus_add_watch_full() or gst_bus_add_watch(). This function will
- * receive messages a short while after they have been posted.
+ * install a #GSource in the default glib main loop and will deliver messages 
+ * a short while after they have been posted. Note that the main loop should 
+ * be running for the asynchronous callbacks.
  *
  * It is also possible to get messages from the bus without any thread
  * marshalling with the gst_bus_set_sync_handler() method. This makes it
@@ -61,16 +63,16 @@
  * Note that a #GstPipeline will set its bus into flushing state when changing
  * from READY to NULL state.
  *
- * Last reviewed on 2005-10-28 (0.9.4)
+ * Last reviewed on 2006-03-12 (0.10.5)
  */
 
+#include "gst_private.h"
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
 #include <sys/types.h>
 
-#include "gst_private.h"
 #include "gstinfo.h"
 
 #include "gstbus.h"
@@ -100,12 +102,17 @@ static guint gst_bus_signals[LAST_SIGNAL] = { 0 };
 /* the context we wakeup when we posted a message on the bus */
 static GMainContext *main_context;
 
+struct _GstBusPrivate
+{
+  guint num_sync_message_emitters;
+};
+
 GType
 gst_bus_get_type (void)
 {
   static GType bus_type = 0;
 
-  if (!bus_type) {
+  if (G_UNLIKELY (bus_type == 0)) {
     static const GTypeInfo bus_info = {
       sizeof (GstBusClass),
       NULL,
@@ -190,7 +197,8 @@ gst_bus_class_init (GstBusClass * klass)
    * @message: the message that has been posted asynchronously
    *
    * A message has been posted on the bus. This signal is emitted from a
-   * GSource added to the mainloop.
+   * GSource added to the mainloop. this signal will only be emited when
+   * there is a mainloop running.
    */
   gst_bus_signals[ASYNC_MESSAGE] =
       g_signal_new ("message", G_TYPE_FROM_CLASS (klass),
@@ -199,6 +207,8 @@ gst_bus_class_init (GstBusClass * klass)
       marshal_VOID__MINIOBJECT, G_TYPE_NONE, 1, GST_TYPE_MESSAGE);
 
   main_context = g_main_context_default ();
+
+  g_type_class_add_private (klass, sizeof (GstBusPrivate));
 }
 
 static void
@@ -207,6 +217,8 @@ gst_bus_init (GstBus * bus)
   bus->queue = g_queue_new ();
   bus->queue_lock = g_mutex_new ();
 
+  bus->priv = G_TYPE_INSTANCE_GET_PRIVATE (bus, GST_TYPE_BUS, GstBusPrivate);
+
   GST_DEBUG_OBJECT (bus, "created");
 }
 
@@ -301,13 +313,14 @@ gst_bus_post (GstBus * bus, GstMessage * message)
 {
   GstBusSyncReply reply = GST_BUS_PASS;
   GstBusSyncHandler handler;
+  gboolean emit_sync_message;
   gpointer handler_data;
 
   g_return_val_if_fail (GST_IS_BUS (bus), FALSE);
   g_return_val_if_fail (GST_IS_MESSAGE (message), FALSE);
 
   GST_DEBUG_OBJECT (bus, "[msg %p] posting on bus, type %s",
-      message, gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
+      message, GST_MESSAGE_TYPE_NAME (message));
 
   GST_OBJECT_LOCK (bus);
   /* check if the bus is flushing */
@@ -316,12 +329,19 @@ gst_bus_post (GstBus * bus, GstMessage * message)
 
   handler = bus->sync_handler;
   handler_data = bus->sync_handler_data;
+  emit_sync_message = bus->priv->num_sync_message_emitters > 0;
   GST_OBJECT_UNLOCK (bus);
 
   /* first call the sync handler if it is installed */
   if (handler)
     reply = handler (bus, message, handler_data);
 
+  /* emit sync-message if requested to do so via
+     gst_bus_enable_sync_message_emission. terrible but effective */
+  if (emit_sync_message && reply != GST_BUS_DROP
+      && handler != gst_bus_sync_signal_handler)
+    gst_bus_sync_signal_handler (bus, message, NULL);
+
   /* now see what we should do with the message */
   switch (reply) {
     case GST_BUS_DROP:
@@ -473,7 +493,7 @@ gst_bus_pop (GstBus * bus)
   if (message)
     GST_DEBUG_OBJECT (bus, "pop from bus, have %d messages, got message %p, %s",
         g_queue_get_length (bus->queue) + 1, message,
-        gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
+        GST_MESSAGE_TYPE_NAME (message));
   else
     GST_DEBUG_OBJECT (bus, "pop from bus, no messages");
   g_mutex_unlock (bus->queue_lock);
@@ -667,13 +687,14 @@ gst_bus_create_watch (GstBus * bus)
  * @user_data: user data passed to @func.
  * @notify: the function to call when the source is removed.
  *
- * Adds a bus watch to the default main context with the given priority.
- * If the func returns FALSE, the source will be removed.
+ * Adds a bus watch to the default main context with the given @priority.
+ * This function is used to receive asynchronous messages in the main loop.
  *
- * When the func is called, the message belongs to the caller; if you want to
- * keep a copy of it, call gst_message_ref() before leaving the func.
+ * When @func is called, the message belongs to the caller; if you want to
+ * keep a copy of it, call gst_message_ref() before leaving @func.
  *
- * The watch can be removed using #g_source_remove().
+ * The watch can be removed using g_source_remove() or by returning FALSE
+ * from @func.
  *
  * Returns: The event source id.
  *
@@ -709,8 +730,10 @@ gst_bus_add_watch_full (GstBus * bus, gint priority,
  * @user_data: user data passed to @func.
  *
  * Adds a bus watch to the default main context with the default priority.
+ * This function is used to receive asynchronous messages in the main loop.
  *
- * The watch can be removed using #g_source_remove().
+ * The watch can be removed using g_source_remove() or by returning FALSE
+ * from @func.
  *
  * Returns: The event source id.
  *
@@ -857,7 +880,7 @@ gst_bus_poll (GstBus * bus, GstMessageType events, GstClockTimeDiff timeout)
  * @message: the #GstMessage received
  * @data: user data
  *
- * A helper GstBusFunc that can be used to convert all asynchronous messages
+ * A helper #GstBusFunc that can be used to convert all asynchronous messages
  * into signals.
  *
  * Returns: TRUE
@@ -904,14 +927,79 @@ gst_bus_sync_signal_handler (GstBus * bus, GstMessage * message, gpointer data)
   return GST_BUS_PASS;
 }
 
+/**
+ * gst_bus_enable_sync_message_emission:
+ * @bus: a #GstBus on which you want to receive the "sync-message" signal
+ *
+ * Instructs GStreamer to emit the "sync-message" signal after running the bus's
+ * sync handler. This function is here so that code can ensure that they can
+ * synchronously receive messages without having to affect what the bin's sync
+ * handler is. 
+ *
+ * This function may be called multiple times. To clean up, the caller is
+ * responsible for calling gst_bus_disable_sync_message_emission() as many times
+ * as this function is called.
+ *
+ * While this function looks similar to gst_bus_add_signal_watch(), it is not
+ * exactly the same -- this function enables <emphasis>synchronous</emphasis> emission of
+ * signals when messages arrive; gst_bus_add_signal_watch() adds an idle callback
+ * to pop messages off the bus <emphasis>asynchronously</emphasis>. The sync-message signal
+ * comes from the thread of whatever object posted the message; the "message"
+ * signal is marshalled to the main thread via the main loop.
+ *
+ * MT safe.
+ */
+void
+gst_bus_enable_sync_message_emission (GstBus * bus)
+{
+  g_return_if_fail (GST_IS_BUS (bus));
+
+  GST_OBJECT_LOCK (bus);
+
+  bus->priv->num_sync_message_emitters++;
+
+  GST_OBJECT_UNLOCK (bus);
+}
+
+/**
+ * gst_bus_disable_sync_message_emission:
+ * @bus: a #GstBus on which you previously called
+ * gst_bus_enable_sync_message_emission()
+ *
+ * Instructs GStreamer to stop emitting the "sync-message" signal for this bus.
+ * See gst_bus_enable_sync_message_emission() for more information.
+ *
+ * In the event that multiple pieces of code have called
+ * gst_bus_enable_sync_message_emission(), the sync-message emissions will only
+ * be stopped after all calls to gst_bus_enable_sync_message_emission() were
+ * "cancelled" by calling this function. In this way the semantics are exactly
+ * the same as gst_object_ref() that which calls enable should also call
+ * disable.
+ *
+ * MT safe.
+ */
+void
+gst_bus_disable_sync_message_emission (GstBus * bus)
+{
+  g_return_if_fail (GST_IS_BUS (bus));
+
+  g_return_if_fail (bus->num_signal_watchers == 0);
+
+  GST_OBJECT_LOCK (bus);
+
+  bus->priv->num_sync_message_emitters--;
+
+  GST_OBJECT_UNLOCK (bus);
+}
+
 /**
  * gst_bus_add_signal_watch_full:
  * @bus: a #GstBus on which you want to receive the "message" signal
  * @priority: The priority of the watch.
  *
  * Adds a bus signal watch to the default main context with the given priority.
- * After calling this statement, the bus will emit the message signal for each
- * message posted on the bus.
+ * After calling this statement, the bus will emit the "message" signal for each
+ * message posted on the bus when the main loop is running.
  *
  * This function may be called multiple times. To clean up, the caller is
  * responsible for calling gst_bus_remove_signal_watch() as many times as this
@@ -949,7 +1037,7 @@ done:
  *
  * Adds a bus signal watch to the default main context with the default
  * priority.
- * After calling this statement, the bus will emit the message signal for each
+ * After calling this statement, the bus will emit the "message" signal for each
  * message posted on the bus.
  *
  * This function may be called multiple times. To clean up, the caller is
