@@ -117,8 +117,7 @@ gst_g_error_get_type (void)
   return type;
 }
 
-#define FILE_A_BUG "  Please file a bug at "\
-    "http://bugzilla.gnome.org/enter_bug.cgi?product=GStreamer."
+#define FILE_A_BUG "  Please file a bug at " PACKAGE_BUGREPORT "."
 
 /* initialize the dynamic table of translated core errors */
 static gchar **
@@ -233,6 +232,7 @@ _gst_stream_errors_init (void)
   TABLE (t, STREAM, ENCODE, N_("Could not encode stream."));
   TABLE (t, STREAM, DEMUX, N_("Could not demultiplex stream."));
   TABLE (t, STREAM, MUX, N_("Could not multiplex stream."));
+  TABLE (t, STREAM, FORMAT, N_("The stream is in the wrong format."));
 
   return t;
 }
