@@ -232,6 +232,13 @@ gst_mini_object_make_writable (GstMiniObject * mini_object)
  *
  * Increase the reference count of the mini-object.
  *
+ * Note that the refcount affects the writeability
+ * of @mini-object, see gst_mini_object_is_writable(). It is 
+ * important to note that keeping additional references to
+ * GstMiniObject instances can potentially increase the number
+ * of memcpy operations in a pipeline, especially if the minibject
+ * is a #GstBuffer.
+ *
  * Returns: the mini-object.
  */
 GstMiniObject *
@@ -397,11 +404,13 @@ gst_value_mini_object_lcopy (const GValue * value, guint n_collect_values,
 void
 gst_value_set_mini_object (GValue * value, GstMiniObject * mini_object)
 {
+  gpointer *pointer_p;
+
   g_return_if_fail (GST_VALUE_HOLDS_MINI_OBJECT (value));
   g_return_if_fail (mini_object == NULL || GST_IS_MINI_OBJECT (mini_object));
 
-  gst_mini_object_replace ((GstMiniObject **) & value->data[0].v_pointer,
-      mini_object);
+  pointer_p = &value->data[0].v_pointer;
+  gst_mini_object_replace ((GstMiniObject **) pointer_p, mini_object);
 }
 
 /**
@@ -417,12 +426,17 @@ gst_value_set_mini_object (GValue * value, GstMiniObject * mini_object)
 void
 gst_value_take_mini_object (GValue * value, GstMiniObject * mini_object)
 {
+  gpointer *pointer_p;
+
   g_return_if_fail (GST_VALUE_HOLDS_MINI_OBJECT (value));
   g_return_if_fail (mini_object == NULL || GST_IS_MINI_OBJECT (mini_object));
 
-  gst_mini_object_replace ((GstMiniObject **) & value->data[0].v_pointer,
-      mini_object);
-  gst_mini_object_unref (mini_object);
+  pointer_p = &value->data[0].v_pointer;
+  /* takes additional refcount */
+  gst_mini_object_replace ((GstMiniObject **) pointer_p, mini_object);
+  /* remove additional refcount */
+  if (mini_object)
+    gst_mini_object_unref (mini_object);
 }
 
 /**
