@@ -388,6 +388,20 @@ gst_structure_id_set_value (GstStructure * structure,
   g_return_if_fail (G_IS_VALUE (value));
   g_return_if_fail (IS_MUTABLE (structure));
 
+  /* if someones disables GST_DEBUG output, they probably do it for
+   * performance reasons, so skip the UTF-8 check here as well then */
+#ifndef GST_DISABLE_GST_DEBUG
+  if (G_VALUE_HOLDS_STRING (value)) {
+    const gchar *s;
+
+    s = g_value_get_string (value);
+    if (s != NULL && !g_utf8_validate (s, -1, NULL)) {
+      g_warning ("Trying to set string field '%s' on structure, but string is "
+          "not valid UTF-8. Please file a bug.", g_quark_to_string (field));
+    }
+  }
+#endif
+
   gsfield.name = field;
   gst_value_init_and_copy (&gsfield.value, value);
 
@@ -466,12 +480,10 @@ gst_structure_set_valist (GstStructure * structure,
 
     type = va_arg (varargs, GType);
 
-#if GLIB_CHECK_VERSION(2,8,0)
     if (type == G_TYPE_DATE) {
       g_warning ("Don't use G_TYPE_DATE, use GST_TYPE_DATE instead\n");
       type = GST_TYPE_DATE;
     }
-#endif
 
     g_value_init (&field.value, type);
     G_VALUE_COLLECT (&field.value, varargs, 0, &err);
@@ -1215,6 +1227,8 @@ typedef struct _GstStructureAbbreviation
 }
 GstStructureAbbreviation;
 
+/* return a copy of an array of GstStructureAbbreviation containing all the
+ * known type_string, GType maps, including abbreviations for common types */
 static GstStructureAbbreviation *
 gst_structure_get_abbrs (gint * n_abbrs)
 {
@@ -1266,8 +1280,10 @@ gst_structure_get_abbrs (gint * n_abbrs)
   return abbrs;
 }
 
+/* given a type_name that could be a type abbreviation or a registered GType,
+ * return a matching GType */
 static GType
-gst_structure_from_abbr (const char *type_name)
+gst_structure_gtype_from_abbr (const char *type_name)
 {
   int i;
   GstStructureAbbreviation *abbrs;
@@ -1628,6 +1644,7 @@ gst_structure_parse_value (gchar * str,
   while (g_ascii_isspace (*s))
     s++;
 
+  /* check if there's a (type_name) 'cast' */
   type_name = NULL;
   if (*s == '(') {
     type = G_TYPE_INVALID;
@@ -1649,7 +1666,7 @@ gst_structure_parse_value (gchar * str,
 
     c = *type_end;
     *type_end = 0;
-    type = gst_structure_from_abbr (type_name);
+    type = gst_structure_gtype_from_abbr (type_name);
     *type_end = c;
 
     if (type == G_TYPE_INVALID)
