@@ -21,8 +21,9 @@
 #define __GST_STRUCTURE_H__
 
 #include <gst/gstconfig.h>
-#include <gst/gsttypes.h>
 #include <glib-object.h>
+#include <gst/gstclock.h>
+#include <gst/glib-compat.h>
 
 G_BEGIN_DECLS
 
@@ -32,21 +33,59 @@ G_BEGIN_DECLS
 
 typedef struct _GstStructure GstStructure;
 
+/**
+ * GstStructureForeachFunc:
+ * @field_id: the #GQuark of the field name
+ * @value: the #GValue of the field
+ * @user_data: user data
+ *
+ * A function that will be called in gst_structure_foreach(). The function may
+ * not modify @value.
+ *
+ * Returns: TRUE if the foreach operation should continue, FALSE if
+ * the foreach operation should stop with FALSE.
+ */
 typedef gboolean (*GstStructureForeachFunc) (GQuark   field_id,
+					     const GValue * value,
+					     gpointer user_data);
+
+/**
+ * GstStructureMapFunc:
+ * @field_id: the #GQuark of the field name
+ * @value: the #GValue of the field
+ * @user_data: user data
+ *
+ * A function that will be called in gst_structure_map_in_place(). The function
+ * may modify @value.
+ *
+ * Returns: TRUE if the map operation should continue, FALSE if
+ * the map operation should stop with FALSE.
+ */
+typedef gboolean (*GstStructureMapFunc)     (GQuark   field_id,
 					     GValue * value,
 					     gpointer user_data);
 
+/**
+ * GstStructure:
+ * @type: the GType of a structure
+ *
+ * The GstStructure object. Most fields are private.
+ */
 struct _GstStructure {
   GType type;
 
+  /*< private >*/
   GQuark name;
 
+  /* owned by parent structure, NULL if no parent */
+  gint *parent_refcount;
+
   GArray *fields;
 
-  gpointer _gst_reserved[GST_PADDING];
+  gpointer _gst_reserved;
 };
 
-GType                   gst_structure_get_type             (void) G_GNUC_CONST;
+GType                   gst_structure_get_type             (void);
 
 GstStructure *          gst_structure_empty_new            (const gchar *            name);
 GstStructure *          gst_structure_id_empty_new         (GQuark                   quark);
@@ -57,10 +96,14 @@ GstStructure *          gst_structure_new_valist           (const gchar *
 						            const gchar *            firstfield,
 							    va_list                  varargs);
 GstStructure *          gst_structure_copy                 (const GstStructure      *structure);
+void			gst_structure_set_parent_refcount  (GstStructure            *structure,
+                                                            gint            *refcount);
 void                    gst_structure_free                 (GstStructure            *structure);
 
 G_CONST_RETURN gchar *  gst_structure_get_name             (const GstStructure      *structure);
 GQuark			gst_structure_get_name_id          (const GstStructure      *structure);
+gboolean                gst_structure_has_name             (const GstStructure      *structure,
+							    const gchar             *name);
 void                    gst_structure_set_name             (GstStructure            *structure,
 							    const gchar             *name);
 
@@ -72,7 +115,7 @@ void                    gst_structure_set_value            (GstStructure
 							    const GValue            *value);
 void                    gst_structure_set                  (GstStructure            *structure,
 							    const gchar             *fieldname,
-							    ...);
+							    ...) G_GNUC_NULL_TERMINATED;
 void                    gst_structure_set_valist           (GstStructure            *structure,
 							    const gchar             *fieldname,
 							    va_list varargs);
@@ -82,20 +125,24 @@ G_CONST_RETURN GValue * gst_structure_get_value            (const GstStructure
 							    const gchar             *fieldname);
 void                    gst_structure_remove_field         (GstStructure            *structure,
 							    const gchar             *fieldname);
-void                    gst_structure_remove_fields        (GstStructure            *structure, 
+void                    gst_structure_remove_fields        (GstStructure            *structure,
 							     const gchar            *fieldname,
-							    ...);
-void                    gst_structure_remove_fields_valist (GstStructure             *structure, 
+							    ...) G_GNUC_NULL_TERMINATED;
+void                    gst_structure_remove_fields_valist (GstStructure             *structure,
 							    const gchar             *fieldname,
 							    va_list                  varargs);
 void                    gst_structure_remove_all_fields    (GstStructure            *structure);
 
 GType                   gst_structure_get_field_type       (const GstStructure      *structure,
 							    const gchar             *fieldname);
-gboolean                gst_structure_foreach              (GstStructure            *structure,
+gboolean                gst_structure_foreach              (const GstStructure      *structure,
 							    GstStructureForeachFunc  func,
 							    gpointer                 user_data);
+gboolean                gst_structure_map_in_place         (GstStructure            *structure,
+							    GstStructureMapFunc      func,
+							    gpointer                 user_data);
 gint                    gst_structure_n_fields             (const GstStructure      *structure);
+const gchar *           gst_structure_nth_field_name       (const GstStructure      *structure, guint index);
 gboolean                gst_structure_has_field            (const GstStructure      *structure,
 							    const gchar             *fieldname);
 gboolean                gst_structure_has_field_typed      (const GstStructure      *structure,
@@ -115,13 +162,42 @@ gboolean                gst_structure_get_fourcc           (const GstStructure
 gboolean                gst_structure_get_double           (const GstStructure      *structure,
 							    const gchar             *fieldname,
 							    gdouble                 *value);
+gboolean                gst_structure_get_date             (const GstStructure      *structure,
+							    const gchar             *fieldname,
+                                                            GDate                  **value);
+gboolean                gst_structure_get_clock_time       (const GstStructure      *structure,
+							    const gchar             *fieldname,
+							    GstClockTime            *value);
 G_CONST_RETURN gchar *  gst_structure_get_string           (const GstStructure      *structure,
 							    const gchar             *fieldname);
+gboolean                gst_structure_get_enum             (const GstStructure      *structure,
+							    const gchar             *fieldname,
+							    GType                    enumtype,
+							    gint                    *value);
+gboolean                gst_structure_get_fraction         (const GstStructure      *structure,
+							    const gchar             *fieldname,
+							    gint *value_numerator,
+							    gint *value_denominator);
 
 gchar *                 gst_structure_to_string            (const GstStructure      *structure);
 GstStructure *          gst_structure_from_string          (const gchar             *string,
 							    gchar                  **end);
 
+gboolean                 gst_structure_fixate_field_nearest_int    (GstStructure *structure,
+									 const char   *field_name,
+									 int           target);
+gboolean                 gst_structure_fixate_field_nearest_double (GstStructure *structure,
+									 const char   *field_name,
+									 double        target);
+
+gboolean                 gst_structure_fixate_field_boolean (GstStructure *structure,
+									 const char   *field_name,
+									 gboolean        target);
+gboolean                 gst_structure_fixate_field_nearest_fraction (GstStructure *structure,
+									 const char   *field_name,
+									 const gint target_numerator,
+									 const gint target_denominator);
+
 G_END_DECLS
 
 #endif
