@@ -28,10 +28,10 @@
  * When defining own plugins, use the GST_BOILERPLATE ease gobject creation.
  */
 
+#include "gst_private.h"
 #include <stdio.h>
 #include <string.h>
 
-#include "gst_private.h"
 #include "gstghostpad.h"
 #include "gstutils.h"
 #include "gstinfo.h"
@@ -482,6 +482,7 @@ overflow:
  * This function can potentially be very slow if denom > G_MAXUINT32.
  *
  * Returns: @val * @num / @denom, trying to avoid overflows.
+ * In the case of an overflow, this function returns G_MAXUINT64.
  */
 guint64
 gst_util_uint64_scale (guint64 val, guint64 num, guint64 denom)
@@ -528,7 +529,8 @@ do_int64:
  *
  * @num and @denom must be positive integers. @denom cannot be 0.
  *
- * Returns: @val * @num / @denom, avoiding overflow and loss of precision
+ * Returns: @val * @num / @denom, avoiding overflow and loss of precision.
+ * In the case of an overflow, this function returns G_MAXUINT64.
  */
 guint64
 gst_util_uint64_scale_int (guint64 val, gint num, gint denom)
@@ -1437,17 +1439,13 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
   if (srcpadname) {
     GST_CAT_DEBUG (GST_CAT_ELEMENT_PADS, "no link possible from %s:%s to %s",
         GST_DEBUG_PAD_NAME (srcpad), GST_ELEMENT_NAME (dest));
-    if (srcpad)
-      gst_object_unref (srcpad);
-    srcpad = NULL;
     if (destpad)
       gst_object_unref (destpad);
     destpad = NULL;
-  } else {
-    if (srcpad)
-      gst_object_unref (srcpad);
-    srcpad = NULL;
   }
+  if (srcpad)
+    gst_object_unref (srcpad);
+  srcpad = NULL;
 
   if (destpad) {
     /* loop through the existing pads in the destination */
@@ -1463,8 +1461,6 @@ gst_element_link_pads (GstElement * src, const gchar * srcpadname,
               GST_DEBUG_PAD_NAME (temp), GST_DEBUG_PAD_NAME (destpad));
           gst_object_unref (temp);
           gst_object_unref (destpad);
-          if (srcpad)
-            gst_object_unref (srcpad);
           return TRUE;
         }
         if (temp) {
@@ -1920,6 +1916,40 @@ gst_element_query_convert (GstElement * element, GstFormat src_format,
   return ret;
 }
 
+/**
+ * gst_element_seek_simple
+ * @element: a #GstElement to seek on
+ * @format: a #GstFormat to execute the seek in, such as #GST_FORMAT_TIME
+ * @seek_flags: seek options
+ * @seek_pos: position to seek to (relative to the start); if you are doing
+ *            a seek in #GST_FORMAT_TIME this value is in nanoseconds -
+ *            multiply with #GST_SECOND to convert seconds to nanoseconds or
+ *            with #GST_MSECOND to convert milliseconds to nanoseconds.
+ *
+ * Simple API to perform a seek on the given element, meaning it just seeks
+ * to the given position relative to the start of the stream. For more complex
+ * operations like segment seeks (e.g. for looping) or changing the playback
+ * rate or seeking relative to the current position or seeking relative to
+ * the end of the stream you should use gst_element_seek ().
+ *
+ * Note that seeking is usually only possible in PAUSED or PLAYING state.
+ *
+ * Returns: TRUE if the seek operation succeeded (the seek
+ *          might not always be executed instantly though)
+ *
+ * Since: 0.10.7
+ */
+gboolean
+gst_element_seek_simple (GstElement * element, GstFormat format,
+    GstSeekFlags seek_flags, gint64 seek_pos)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+  g_return_val_if_fail (seek_pos >= 0, FALSE);
+
+  return gst_element_seek (element, 1.0, format, seek_flags,
+      GST_SEEK_TYPE_SET, seek_pos, GST_SEEK_TYPE_NONE, 0);
+}
+
 /**
  * gst_pad_can_link:
  * @srcpad: the source #GstPad to link.
@@ -2325,7 +2355,7 @@ gst_buffer_join (GstBuffer * buf1, GstBuffer * buf2)
  * @dest: buffer to stamp
  * @src: buffer to stamp from
  *
- * Copies additional information (the timestamp, duration, and offset start 
+ * Copies additional information (the timestamp, duration, and offset start
  * and end) from one buffer to the other.
  *
  * This function does not copy any buffer flags or caps.
@@ -2531,6 +2561,37 @@ gst_pad_query_position (GstPad * pad, GstFormat * format, gint64 * cur)
   return ret;
 }
 
+/**
+ * gst_pad_query_peer_position:
+ * @pad: a #GstPad on whose peer to invoke the position query on.
+ *       Must be a sink pad.
+ * @format: a pointer to the #GstFormat asked for.
+ *          On return contains the #GstFormat used.
+ * @cur: A location in which to store the current position, or NULL.
+ *
+ * Queries the peer of a given sink pad for the stream position.
+ *
+ * Returns: TRUE if the query could be performed.
+ */
+gboolean
+gst_pad_query_peer_position (GstPad * pad, GstFormat * format, gint64 * cur)
+{
+  gboolean ret = FALSE;
+  GstPad *peer;
+
+  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
+  g_return_val_if_fail (GST_PAD_IS_SINK (pad), FALSE);
+  g_return_val_if_fail (format != NULL, FALSE);
+
+  peer = gst_pad_get_peer (pad);
+  if (peer) {
+    ret = gst_pad_query_position (peer, format, cur);
+    gst_object_unref (peer);
+  }
+
+  return ret;
+}
+
 /**
  * gst_pad_query_duration:
  * @pad: a #GstPad to invoke the duration query on.
@@ -2562,6 +2623,38 @@ gst_pad_query_duration (GstPad * pad, GstFormat * format, gint64 * duration)
   return ret;
 }
 
+/**
+ * gst_pad_query_peer_duration:
+ * @pad: a #GstPad on whose peer pad to invoke the duration query on.
+ *       Must be a sink pad.
+ * @format: a pointer to the #GstFormat asked for.
+ *          On return contains the #GstFormat used.
+ * @duration: A location in which to store the total duration, or NULL.
+ *
+ * Queries the peer pad of a given sink pad for the total stream duration.
+ *
+ * Returns: TRUE if the query could be performed.
+ */
+gboolean
+gst_pad_query_peer_duration (GstPad * pad, GstFormat * format,
+    gint64 * duration)
+{
+  gboolean ret = FALSE;
+  GstPad *peer;
+
+  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
+  g_return_val_if_fail (GST_PAD_IS_SINK (pad), FALSE);
+  g_return_val_if_fail (format != NULL, FALSE);
+
+  peer = gst_pad_get_peer (pad);
+  if (peer) {
+    ret = gst_pad_query_duration (peer, format, duration);
+    gst_object_unref (peer);
+  }
+
+  return ret;
+}
+
 /**
  * gst_pad_query_convert:
  * @pad: a #GstPad to invoke the convert query on.
@@ -2602,6 +2695,43 @@ gst_pad_query_convert (GstPad * pad, GstFormat src_format, gint64 src_val,
   return ret;
 }
 
+/**
+ * gst_pad_query_peer_convert:
+ * @pad: a #GstPad, on whose peer pad to invoke the convert query on.
+ *       Must be a sink pad.
+ * @src_format: a #GstFormat to convert from.
+ * @src_val: a value to convert.
+ * @dest_format: a pointer to the #GstFormat to convert to.
+ * @dest_val: a pointer to the result.
+ *
+ * Queries the peer pad of a given sink pad to convert @src_val in @src_format
+ * to @dest_format.
+ *
+ * Returns: TRUE if the query could be performed.
+ */
+gboolean
+gst_pad_query_peer_convert (GstPad * pad, GstFormat src_format, gint64 src_val,
+    GstFormat * dest_format, gint64 * dest_val)
+{
+  gboolean ret = FALSE;
+  GstPad *peer;
+
+  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
+  g_return_val_if_fail (GST_PAD_IS_SINK (pad), FALSE);
+  g_return_val_if_fail (src_val >= 0, FALSE);
+  g_return_val_if_fail (dest_format != NULL, FALSE);
+  g_return_val_if_fail (dest_val != NULL, FALSE);
+
+  peer = gst_pad_get_peer (pad);
+  if (peer) {
+    ret = gst_pad_query_convert (peer, src_format, src_val, dest_format,
+        dest_val);
+    gst_object_unref (peer);
+  }
+
+  return ret;
+}
+
 /**
  * gst_atomic_int_set:
  * @atomic_int: pointer to an atomic integer
@@ -2627,9 +2757,10 @@ gst_atomic_int_set (gint * atomic_int, gint value)
  *
  * Adds a "data probe" to a pad. This function will be called whenever data
  * passes through a pad. In this case data means both events and buffers. The
- * probe will be called with the data as an argument. Note that the data will
- * have a reference count greater than 1, so it will be immutable -- you must
- * not change it.
+ * probe will be called with the data as an argument, meaning @handler should
+ * have the same callback signature as the 'have-data' signal of #GstPad.
+ * Note that the data will have a reference count greater than 1, so it will
+ * be immutable -- you must not change it.
  *
  * For source pads, the probe will be called after the blocking function, if any
  * (see gst_pad_set_blocked_async()), but before looking up the peer to chain
@@ -2652,10 +2783,19 @@ gulong
 gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
+  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
+  if (GST_IS_GHOST_PAD (pad)) {
+    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
+    if (!pad) {
+      return 0;
+    }
+    was_ghost = TRUE;
+  }
+
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2665,6 +2805,10 @@ gst_pad_add_data_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_PAD_DO_BUFFER_SIGNALS (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
+  if (was_ghost) {
+    gst_object_unref (pad);
+  }
+
   return sigid;
 }
 
@@ -2683,10 +2827,19 @@ gulong
 gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
+  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
+  if (GST_IS_GHOST_PAD (pad)) {
+    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
+    if (!pad) {
+      return 0;
+    }
+    was_ghost = TRUE;
+  }
+
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::event", handler, data);
   GST_PAD_DO_EVENT_SIGNALS (pad)++;
@@ -2694,6 +2847,10 @@ gst_pad_add_event_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
+  if (was_ghost) {
+    gst_object_unref (pad);
+  }
+
   return sigid;
 }
 
@@ -2712,10 +2869,19 @@ gulong
 gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 {
   gulong sigid;
+  gboolean was_ghost = FALSE;
 
   g_return_val_if_fail (GST_IS_PAD (pad), 0);
   g_return_val_if_fail (handler != NULL, 0);
 
+  if (GST_IS_GHOST_PAD (pad)) {
+    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
+    if (!pad) {
+      return 0;
+    }
+    was_ghost = TRUE;
+  }
+
   GST_OBJECT_LOCK (pad);
   sigid = g_signal_connect (pad, "have-data::buffer", handler, data);
   GST_PAD_DO_BUFFER_SIGNALS (pad)++;
@@ -2723,6 +2889,10 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
 
+  if (was_ghost) {
+    gst_object_unref (pad);
+  }
+
   return sigid;
 }
 
@@ -2736,9 +2906,19 @@ gst_pad_add_buffer_probe (GstPad * pad, GCallback handler, gpointer data)
 void
 gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 {
+  gboolean was_ghost = FALSE;
+
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
+  if (GST_IS_GHOST_PAD (pad)) {
+    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
+    if (!pad) {
+      return;
+    }
+    was_ghost = TRUE;
+  }
+
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
@@ -2748,6 +2928,10 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad),
       GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
+
+  if (was_ghost) {
+    gst_object_unref (pad);
+  }
 }
 
 /**
@@ -2760,15 +2944,29 @@ gst_pad_remove_data_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 {
+  gboolean was_ghost = FALSE;
+
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
+  if (GST_IS_GHOST_PAD (pad)) {
+    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
+    if (!pad) {
+      return;
+    }
+    was_ghost = TRUE;
+  }
+
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_EVENT_SIGNALS (pad)--;
   GST_DEBUG ("removed event probe from pad %s:%s, now %d event probes",
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_EVENT_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
+
+  if (was_ghost) {
+    gst_object_unref (pad);
+  }
 }
 
 /**
@@ -2781,15 +2979,29 @@ gst_pad_remove_event_probe (GstPad * pad, guint handler_id)
 void
 gst_pad_remove_buffer_probe (GstPad * pad, guint handler_id)
 {
+  gboolean was_ghost = FALSE;
+
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (handler_id > 0);
 
+  if (GST_IS_GHOST_PAD (pad)) {
+    pad = gst_ghost_pad_get_target (GST_GHOST_PAD (pad));
+    if (!pad) {
+      return;
+    }
+    was_ghost = TRUE;
+  }
+
   GST_OBJECT_LOCK (pad);
   g_signal_handler_disconnect (pad, handler_id);
   GST_PAD_DO_BUFFER_SIGNALS (pad)--;
   GST_DEBUG ("removed buffer probe from pad %s:%s, now %d buffer probes",
       GST_DEBUG_PAD_NAME (pad), GST_PAD_DO_BUFFER_SIGNALS (pad));
   GST_OBJECT_UNLOCK (pad);
+
+  if (was_ghost) {
+    gst_object_unref (pad);
+  }
 }
 
 /**
@@ -2821,6 +3033,8 @@ static void
 push_and_ref (GstPad * pad, GstEvent * event)
 {
   gst_pad_push_event (pad, gst_event_ref (event));
+  /* iterator refs pad, we unref when we are done with it */
+  gst_object_unref (pad);
 }
 
 /**
@@ -2966,6 +3180,7 @@ gst_bin_find_unconnected_pad (GstBin * bin, GstPadDirection direction)
   return pad;
 }
 
+#ifndef GST_DISABLE_PARSE
 /**
  * gst_parse_bin_from_description:
  * @bin_description: command line describing the bin
@@ -3026,3 +3241,4 @@ gst_parse_bin_from_description (const gchar * bin_description,
 
   return GST_ELEMENT (bin);
 }
+#endif
