@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) 2004 Benjamin Otte <otte@gnome.org>
+ *               2005 Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +28,9 @@
  * a file. So if you have undefined buffer sizes and require a specific size,
  * this object is for you.
  *
+ * An adapter is created with gst_adapter_new(). It can be freed again with
+ * g_object_unref().
+ *
  * The theory of operation is like this: All buffers received are put
  * into the adapter using gst_adapter_push() and the data is then read back
  * in chunks of the desired size using gst_adapter_peek(). After the data is
@@ -86,12 +90,12 @@
  * access the buffer later. The adapter will never modify the data in the
  * buffer pushed in it.
  *
- * Last reviewed on 2006-03-07 (0.10.4).
+ * Last reviewed on 2006-04-04 (0.10.6).
  */
 
-#include <string.h>
 
 #include "gstadapter.h"
+#include <string.h>
 
 /* default size for the assembled data buffer */
 #define DEFAULT_SIZE 16
@@ -152,7 +156,7 @@ gst_adapter_finalize (GObject * object)
 /**
  * gst_adapter_new:
  *
- * Creates a new #GstAdapter.
+ * Creates a new #GstAdapter. Free with g_object_unref().
  *
  * Returns: a new #GstAdapter
  */
@@ -196,6 +200,8 @@ gst_adapter_push (GstAdapter * adapter, GstBuffer * buf)
   g_return_if_fail (GST_IS_BUFFER (buf));
 
   adapter->size += GST_BUFFER_SIZE (buf);
+  /* FIXME, _append does not scale. Note: merging buffers at this
+   * point is premature. */
   adapter->buflist = g_slist_append (adapter->buflist, buf);
 }
 
@@ -213,7 +219,8 @@ gst_adapter_push (GstAdapter * adapter, GstBuffer * buf)
  * of its chain function, the buffer will have an invalid data pointer after
  * your element flushes the bytes. In that case you should use
  * gst_adapter_take(), which returns a freshly-allocated buffer that you can set
- * as #GstBuffer malloc_data.
+ * as #GstBuffer malloc_data or the potentially more performant 
+ * gst_adapter_take_buffer().
  *
  * Returns #NULL if @size bytes are not available.
  *
@@ -229,8 +236,10 @@ gst_adapter_peek (GstAdapter * adapter, guint size)
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), NULL);
   g_return_val_if_fail (size > 0, NULL);
 
-  /* we don't have enough data, return NULL */
-  if (size > adapter->size)
+  /* we don't have enough data, return NULL. This is unlikely
+   * as one usually does an _available() first instead of peeking a
+   * random size. */
+  if (G_UNLIKELY (size > adapter->size))
     return NULL;
 
   /* we have enough assembled data, return it */
@@ -253,6 +262,7 @@ gst_adapter_peek (GstAdapter * adapter, guint size)
   copied = GST_BUFFER_SIZE (cur) - adapter->skip;
   memcpy (adapter->assembled_data, GST_BUFFER_DATA (cur) + adapter->skip,
       copied);
+
   cur_list = g_slist_next (adapter->buflist);
   while (copied < size) {
     g_assert (cur_list);
@@ -327,7 +337,6 @@ gst_adapter_take (GstAdapter * adapter, guint nbytes)
   GST_LOG_OBJECT (adapter, "taking %u bytes", nbytes);
 
   cdata = gst_adapter_peek (adapter, nbytes);
-
   if (!cdata)
     return NULL;
 
@@ -339,6 +348,48 @@ gst_adapter_take (GstAdapter * adapter, guint nbytes)
   return data;
 }
 
+/**
+ * gst_adapter_take_buffer:
+ * @adapter: a #GstAdapter
+ * @nbytes: the number of bytes to take
+ *
+ * Returns a #GstBuffer containing the first @nbytes bytes of the
+ * @adapter. The returned bytes will be flushed from the adapter.
+ * This function is potentially more performant that gst_adapter_take() 
+ * since it can reuse the memory in the pushed buffer by subbuffering
+ * or merging.
+ *
+ * Caller owns returned value. gst_buffer_unref() after usage.
+ *
+ * Since: 0.10.6
+ *
+ * Returns: a #GstBuffer containing the first @nbytes of the adapter, 
+ * or #NULL if @nbytes bytes are not available
+ */
+GstBuffer *
+gst_adapter_take_buffer (GstAdapter * adapter, guint nbytes)
+{
+  GstBuffer *buffer;
+  guint8 *data;
+
+  g_return_val_if_fail (GST_IS_ADAPTER (adapter), NULL);
+  g_return_val_if_fail (nbytes > 0, NULL);
+
+  GST_LOG_OBJECT (adapter, "taking buffer of %u bytes", nbytes);
+
+  /* FIXME, optimize me */
+  data = gst_adapter_take (adapter, nbytes);
+  if (data == NULL)
+    return NULL;
+
+  buffer = gst_buffer_new ();
+  GST_BUFFER_DATA (buffer) = data;
+  GST_BUFFER_MALLOCDATA (buffer) = data;
+  GST_BUFFER_SIZE (buffer) = nbytes;
+
+  return buffer;
+}
+
 /**
  * gst_adapter_available:
  * @adapter: a #GstAdapter
@@ -371,10 +422,17 @@ gst_adapter_available_fast (GstAdapter * adapter)
 {
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), 0);
 
+  /* no buffers, we have no data */
   if (!adapter->buflist)
     return 0;
+
+  /* some stuff we already assembled */
   if (adapter->assembled_len)
     return adapter->assembled_len;
+
+  /* we cannot have skipped more than the first buffer */
   g_assert (GST_BUFFER_SIZE (adapter->buflist->data) > adapter->skip);
+
+  /* we can quickly get the data of the first buffer */
   return GST_BUFFER_SIZE (adapter->buflist->data) - adapter->skip;
 }
