@@ -142,6 +142,7 @@ gst_collect_pads_finalize (GObject * object)
   }
   /* Free pads list */
   g_slist_free (pads->data);
+  g_slist_free (pads->abidata.ABI.pad_list);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -196,7 +197,9 @@ gst_collect_pads_set_function (GstCollectPads * pads,
  * @size: the size of the returned GstCollectData structure
  *
  * Add a pad to the collection of collect pads. The pad has to be
- * a sinkpad.
+ * a sinkpad. The refcount of the pad is incremented. Use
+ * gst_collect_pads_remove_pad() to remove the pad from the collection
+ * again.
  *
  * You specify a size for the returned #GstCollectData structure
  * so that you can use it to store additional information.
@@ -301,6 +304,8 @@ gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
   pads->abidata.ABI.pad_list =
       g_slist_delete_link (pads->abidata.ABI.pad_list, list);
   pads->abidata.ABI.pad_cookie++;
+  /* signal waiters because something changed */
+  GST_COLLECT_PADS_BROADCAST (pads);
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
 
   return TRUE;
@@ -308,6 +313,7 @@ gst_collect_pads_remove_pad (GstCollectPads * pads, GstPad * pad)
 unknown_pad:
   {
     GST_WARNING ("cannot remove unknown pad %s:%s", GST_DEBUG_PAD_NAME (pad));
+    GST_COLLECT_PADS_PAD_UNLOCK (pads);
     return FALSE;
   }
 }
@@ -395,7 +401,8 @@ gst_collect_pads_collect_range (GstCollectPads * pads, guint64 offset,
  * Must be called with PAD_LOCK.
  */
 static void
-gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
+gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
+    gboolean flushing)
 {
   GSList *walk = NULL;
 
@@ -409,11 +416,36 @@ gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
         GST_PAD_SET_FLUSHING (cdata->pad);
       else
         GST_PAD_UNSET_FLUSHING (cdata->pad);
+      cdata->abidata.ABI.flushing = flushing;
       GST_OBJECT_UNLOCK (cdata->pad);
     }
   }
 }
 
+/**
+ * gst_collect_pads_set_flushing:
+ * @pads: the collectspads to use
+ * @flushing: desired state of the pads
+ *
+ * Change the flushing state of all the pads in the collection. No pad
+ * is able to accept anymore data when @flushing is %TRUE. Calling this
+ * function with @flushing %TRUE makes @pads accept data again.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.7.
+ */
+void
+gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
+{
+  g_return_if_fail (pads != NULL);
+  g_return_if_fail (GST_IS_COLLECT_PADS (pads));
+
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  gst_collect_pads_set_flushing_unlocked (pads, flushing);
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
+}
+
 /**
  * gst_collect_pads_start:
  * @pads: the collectspads to use
@@ -425,6 +457,8 @@ gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
 void
 gst_collect_pads_start (GstCollectPads * pads)
 {
+  GSList *collected;
+
   g_return_if_fail (pads != NULL);
   g_return_if_fail (GST_IS_COLLECT_PADS (pads));
 
@@ -435,7 +469,17 @@ gst_collect_pads_start (GstCollectPads * pads)
 
   /* make pads streamable */
   GST_COLLECT_PADS_PAD_LOCK (pads);
-  gst_collect_pads_set_flushing (pads, FALSE);
+
+  /* loop over the master pad list and reset the segment */
+  collected = pads->abidata.ABI.pad_list;
+  for (; collected; collected = g_slist_next (collected)) {
+    GstCollectData *data;
+
+    data = collected->data;
+    gst_segment_init (&data->segment, GST_FORMAT_UNDEFINED);
+  }
+
+  gst_collect_pads_set_flushing_unlocked (pads, FALSE);
 
   /* Start collect pads */
   pads->started = TRUE;
@@ -455,6 +499,8 @@ gst_collect_pads_start (GstCollectPads * pads)
 void
 gst_collect_pads_stop (GstCollectPads * pads)
 {
+  GSList *collected;
+
   g_return_if_fail (pads != NULL);
   g_return_if_fail (GST_IS_COLLECT_PADS (pads));
 
@@ -465,10 +511,28 @@ gst_collect_pads_stop (GstCollectPads * pads)
 
   /* make pads not accept data anymore */
   GST_COLLECT_PADS_PAD_LOCK (pads);
-  gst_collect_pads_set_flushing (pads, TRUE);
+  gst_collect_pads_set_flushing_unlocked (pads, TRUE);
 
   /* Stop collect pads */
   pads->started = FALSE;
+  pads->eospads = 0;
+  pads->queuedpads = 0;
+
+  /* loop over the master pad list and flush buffers */
+  collected = pads->abidata.ABI.pad_list;
+  for (; collected; collected = g_slist_next (collected)) {
+    GstCollectData *data;
+    GstBuffer **buffer_p;
+
+    data = collected->data;
+    if (data->buffer) {
+      buffer_p = &data->buffer;
+      gst_buffer_replace (buffer_p, NULL);
+      data->pos = 0;
+    }
+    data->abidata.ABI.eos = FALSE;
+  }
+
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
   /* Wake them up so then can end the chain functions. */
   GST_COLLECT_PADS_BROADCAST (pads);
@@ -720,6 +784,7 @@ gst_collect_pads_check_pads (GstCollectPads * pads)
     GSList *collected;
 
     /* clear list and stats */
+    g_slist_free (pads->data);
     pads->data = NULL;
     pads->numpads = 0;
     pads->queuedpads = 0;
@@ -779,8 +844,9 @@ gst_collect_pads_check_collected (GstCollectPads * pads)
      * function
      */
     while (((pads->queuedpads + pads->eospads) >= pads->numpads)) {
-      GST_DEBUG ("All active pads (%d) have data, calling %s",
-          pads->numpads, GST_DEBUG_FUNCPTR_NAME (pads->func));
+      GST_DEBUG ("All active pads (%d + %d >= %d) have data, calling %s",
+          pads->queuedpads, pads->eospads, pads->numpads,
+          GST_DEBUG_FUNCPTR_NAME (pads->func));
       flow_ret = pads->func (pads, pads->user_data);
       collected = TRUE;
 
@@ -840,6 +906,9 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
       GST_OBJECT_LOCK (pads);
       data->abidata.ABI.flushing = FALSE;
       gst_collect_pads_pop (pads, data);
+      /* we need new segment info after the flush */
+      gst_segment_init (&data->segment, GST_FORMAT_UNDEFINED);
+      data->abidata.ABI.new_segment = FALSE;
       /* if the pad was EOS, remove the EOS flag and
        * decrement the number of eospads */
       if (G_UNLIKELY (data->abidata.ABI.eos == TRUE)) {
@@ -979,16 +1048,20 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
       gst_segment_set_last_stop (&data->segment, GST_FORMAT_TIME, timestamp);
   }
 
-  /* Check if our collected condition is matched and call the collected function
-   * if it is */
-  ret = gst_collect_pads_check_collected (pads);
-  /* when an error occurs, we want to report this back to the caller ASAP
-   * without having to block if the buffer was not popped */
-  if (G_UNLIKELY (ret != GST_FLOW_OK))
-    goto error;
+  /* While we have data queued on this pad try to collect stuff */
+  do {
+    /* Check if our collected condition is matched and call the collected function
+     * if it is */
+    ret = gst_collect_pads_check_collected (pads);
+    /* when an error occurs, we want to report this back to the caller ASAP
+     * without having to block if the buffer was not popped */
+    if (G_UNLIKELY (ret != GST_FLOW_OK))
+      goto error;
+
+    /* data was consumed, we can exit and accept new data */
+    if (data->buffer == NULL)
+      break;
 
-  /* We still have data queued on this pad, wait for something to happen */
-  while (data->buffer != NULL) {
     GST_DEBUG ("Pad %s:%s has a buffer queued, waiting",
         GST_DEBUG_PAD_NAME (pad));
 
@@ -1007,6 +1080,8 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
     if (G_UNLIKELY (data->abidata.ABI.flushing))
       goto flushing;
   }
+  while (data->buffer != NULL);
+
   GST_OBJECT_UNLOCK (pads);
 
   return ret;
