@@ -787,7 +787,47 @@ gst_controller_unset (GstController * self, gchar * property_name,
 
   g_mutex_lock (self->lock);
   if ((prop = gst_controller_find_controlled_property (self, property_name))) {
-    prop->values = g_list_remove (prop->values, prop);
+    GList *node;
+
+    /* check if a timed_value for the timestamp exists */
+    if ((node = g_list_find_custom (prop->values, &timestamp,
+                gst_timed_value_find))) {
+      g_free (node->data);      /* free GstTimedValue */
+      prop->values = g_list_delete_link (prop->values, node);
+      res = TRUE;
+    }
+  }
+  g_mutex_unlock (self->lock);
+
+  return (res);
+}
+
+/**
+ * gst_controller_unset_all:
+ * @self: the controller object which handles the properties
+ * @property_name: the name of the property to unset
+ *
+ * Used to remove all time-stamped values of given controller-handled property
+ *
+ * Returns: %FALSE if the values couldn't be unset (ex : properties not handled
+ * by controller), %TRUE otherwise
+ * Since: 0.10.5
+ */
+gboolean
+gst_controller_unset_all (GstController * self, gchar * property_name)
+{
+  gboolean res = FALSE;
+  GstControlledProperty *prop;
+
+  g_return_val_if_fail (GST_IS_CONTROLLER (self), FALSE);
+  g_return_val_if_fail (property_name, FALSE);
+
+  g_mutex_lock (self->lock);
+  if ((prop = gst_controller_find_controlled_property (self, property_name))) {
+    /* free GstTimedValue structures */
+    g_list_foreach (prop->values, (GFunc) g_free, NULL);
+    g_list_free (prop->values);
+    prop->values = NULL;
     res = TRUE;
   }
   g_mutex_unlock (self->lock);
@@ -890,10 +930,10 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
         timestamp);
 
     live = FALSE;
-    if (G_IS_VALUE (&prop->live_value.value)) {
+    if (G_UNLIKELY (G_IS_VALUE (&prop->live_value.value))) {
       GList *lnode =
           gst_controlled_property_find_timed_value_node (prop, timestamp);
-      if (!lnode) {
+      if (G_UNLIKELY (!lnode)) {
         GST_DEBUG ("    no control changes in the queue");
         live = TRUE;
       } else {
@@ -907,7 +947,7 @@ gst_controller_sync_values (GstController * self, GstClockTime timestamp)
         }
       }
     }
-    if (!live) {
+    if (G_LIKELY (!live)) {
       /* get current value via interpolator */
       value = prop->get (prop, timestamp);
       prop->last_value.timestamp = timestamp;
@@ -1101,7 +1141,7 @@ _gst_controller_class_init (GstControllerClass * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_ref (G_TYPE_OBJECT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = _gst_controller_finalize;
 
