@@ -1,6 +1,6 @@
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
- * Copyright (C) <2004> Thomas Vander Stichele <thomas at apestaart dot org>
+ * Copyright (C) 2004,2006 Thomas Vander Stichele <thomas at apestaart dot org>
  *
  * dataprotocol.c: Functions implementing the GStreamer Data Protocol
  *
@@ -20,6 +20,48 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:gstdataprotocol
+ * @short_description: Serialization of caps, buffers and events.
+ * @see_also: #GstCaps, #GstEvent, #GstBuffer
+ *
+ * This helper library provides serialization of GstBuffer, GstCaps and
+ * GstEvent structures.
+ *
+ * This serialization is useful when GStreamer needs to interface with
+ * the outside world to transport data between distinct GStreamer pipelines.
+ * The connections with the outside world generally don't have mechanisms
+ * to transport properties of these structures.
+ *
+ * For example, transporting buffers across named pipes or network connections
+ * doesn't maintain the buffer size and separation.
+ *
+ * This data protocol assumes a reliable connection-oriented transport, such as
+ * TCP, a pipe, or a file.  The protocol does not serialize the caps for
+ * each buffer; instead, it transport the caps only when they change in the
+ * stream.  This implies that there will always be a caps packet before any
+ * buffer packets.
+ *
+ * The versioning of the protocol is independent of GStreamer's version.
+ * The major number gets incremented, and the minor reset, for incompatible
+ * changes.  The minor number gets incremented for compatible changes that
+ * allow clients who do not completely understand the newer protocol version
+ * to still decode what they do understand.
+ *
+ * Version 0.2 serializes only a small subset of all events, with a custom
+ * payload for each type.  Also, all GDP streams start with the initial caps
+ * packet.
+ *
+ * Version 1.0 serializes all events by taking the string representation of
+ * the event as the payload.  In addition, GDP streams can now start with
+ * events as well, as required by the new data stream model in GStreamer 0.10.
+ *
+ * Converting buffers, caps and events to GDP buffers is done using a
+ * #GstDPPacketizer object and invoking its packetizer functions.
+ * For backwards-compatibility reasons, the old 0.2 methods are still
+ * available but deprecated.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -34,6 +76,40 @@
 GST_DEBUG_CATEGORY (data_protocol_debug);
 #define GST_CAT_DEFAULT data_protocol_debug
 
+/* helper macros */
+
+/* write first 6 bytes of header, as well as ABI padding */
+#define GST_DP_INIT_HEADER(h, version, flags, type)		\
+G_STMT_START {							\
+  gint maj = 0, min = 0;					\
+  switch (version) {						\
+    case GST_DP_VERSION_0_2: maj = 0; min = 2; break;		\
+    case GST_DP_VERSION_1_0: maj = 1; min = 0; break;		\
+  }								\
+  h[0] = (guint8) maj;						\
+  h[1] = (guint8) min;						\
+  h[2] = (guint8) flags;					\
+  h[3] = 0; /* padding byte */					\
+  GST_WRITE_UINT16_BE (h + 4, type);				\
+								\
+  GST_WRITE_UINT64_BE (h + 42, (guint64) 0); /* ABI padding */	\
+  GST_WRITE_UINT64_BE (h + 50, (guint64) 0); /* ABI padding */	\
+} G_STMT_END
+
+#define GST_DP_SET_CRC(h, flags, payload, length);		\
+G_STMT_START {							\
+  guint16 crc = 0;						\
+  if (flags & GST_DP_HEADER_FLAG_CRC_HEADER)			\
+    /* we don't crc the last four bytes since they are crc's */ \
+    crc = gst_dp_crc (h, 58);					\
+  GST_WRITE_UINT16_BE (h + 58, crc);				\
+								\
+  crc = 0;							\
+  if (length && (flags & GST_DP_HEADER_FLAG_CRC_PAYLOAD))	\
+    crc = gst_dp_crc (payload, length);				\
+  GST_WRITE_UINT16_BE (h + 60, crc);				\
+} G_STMT_END
+
 /* calculate a CCITT 16 bit CRC check value for a given byte array */
 /*
  * this code snippet is adapted from a web page I found
@@ -46,8 +122,100 @@ GST_DEBUG_CATEGORY (data_protocol_debug);
 #define POLY       0x1021
 #define CRC_INIT   0xFFFF
 
-static guint16
-gst_dp_crc (const guint8 * buffer, register guint length)
+/*** HELPER FUNCTIONS ***/
+
+static gboolean
+gst_dp_header_from_buffer_any (const GstBuffer * buffer, GstDPHeaderFlag flags,
+    guint * length, guint8 ** header, GstDPVersion version)
+{
+  guint8 *h;
+  guint16 flags_mask;
+
+  g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
+  g_return_val_if_fail (header, FALSE);
+
+  *length = GST_DP_HEADER_LENGTH;
+  h = g_malloc0 (GST_DP_HEADER_LENGTH);
+
+  /* version, flags, type */
+  GST_DP_INIT_HEADER (h, version, flags, GST_DP_PAYLOAD_BUFFER);
+
+  /* buffer properties */
+  GST_WRITE_UINT32_BE (h + 6, GST_BUFFER_SIZE (buffer));
+  GST_WRITE_UINT64_BE (h + 10, GST_BUFFER_TIMESTAMP (buffer));
+  GST_WRITE_UINT64_BE (h + 18, GST_BUFFER_DURATION (buffer));
+  GST_WRITE_UINT64_BE (h + 26, GST_BUFFER_OFFSET (buffer));
+  GST_WRITE_UINT64_BE (h + 34, GST_BUFFER_OFFSET_END (buffer));
+
+  /* data flags; eats two bytes from the ABI area */
+  /* we copy everything but the read-only flags */
+  flags_mask = GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_DISCONT |
+      GST_BUFFER_FLAG_IN_CAPS | GST_BUFFER_FLAG_GAP |
+      GST_BUFFER_FLAG_DELTA_UNIT;
+
+  GST_WRITE_UINT16_BE (h + 42, GST_BUFFER_FLAGS (buffer) & flags_mask);
+
+  GST_DP_SET_CRC (h, flags, GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
+
+  GST_LOG ("created header from buffer:");
+  gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
+  *header = h;
+  return TRUE;
+}
+
+static gboolean
+gst_dp_packet_from_caps_any (const GstCaps * caps, GstDPHeaderFlag flags,
+    guint * length, guint8 ** header, guint8 ** payload, GstDPVersion version)
+{
+  guint8 *h;
+  guchar *string;
+  guint payload_length;
+
+  /* FIXME: GST_IS_CAPS doesn't work
+     g_return_val_if_fail (GST_IS_CAPS (caps), FALSE); */
+  g_return_val_if_fail (caps, FALSE);
+  g_return_val_if_fail (header, FALSE);
+  g_return_val_if_fail (payload, FALSE);
+
+  *length = GST_DP_HEADER_LENGTH;
+  h = g_malloc0 (GST_DP_HEADER_LENGTH);
+
+  string = (guchar *) gst_caps_to_string (caps);
+  payload_length = strlen ((gchar *) string) + 1;       /* include trailing 0 */
+
+  /* version, flags, type */
+  GST_DP_INIT_HEADER (h, version, flags, GST_DP_PAYLOAD_CAPS);
+
+  /* buffer properties */
+  GST_WRITE_UINT32_BE (h + 6, payload_length);
+  GST_WRITE_UINT64_BE (h + 10, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 18, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 26, (guint64) 0);
+  GST_WRITE_UINT64_BE (h + 34, (guint64) 0);
+
+  GST_DP_SET_CRC (h, flags, string, payload_length);
+
+  GST_LOG ("created header from caps:");
+  gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
+  *header = h;
+  *payload = string;
+  return TRUE;
+}
+
+
+/*** PUBLIC FUNCTIONS ***/
+
+/**
+ * gst_dp_crc:
+ *
+ * Calculate a CRC for the given buffer over the given number of bytes.
+ * This is only provided for verification purposes; typical GDP users
+ * will not need this function.
+ *
+ * Returns: a two-byte CRC checksum.
+ */
+guint16
+gst_dp_crc (const guint8 * buffer, guint length)
 {
   static gboolean initialized = FALSE;
   static guint16 crc_table[256];
@@ -96,6 +264,23 @@ gst_dp_dump_byte_array (guint8 * array, guint length)
   g_free (line);
 }
 
+GType
+gst_dp_version_get_type (void)
+{
+  static GType gst_dp_version_type = 0;
+  static const GEnumValue gst_dp_version[] = {
+    {GST_DP_VERSION_0_2, "GDP Version 0.2", "0.2"},
+    {GST_DP_VERSION_1_0, "GDP Version 1.0", "1.0"},
+    {0, NULL, NULL},
+  };
+
+  if (!gst_dp_version_type) {
+    gst_dp_version_type =
+        g_enum_register_static ("GstDPVersion", gst_dp_version);
+  }
+  return gst_dp_version_type;
+};
+
 /**
  * gst_dp_init:
  *
@@ -114,12 +299,12 @@ gst_dp_init (void)
 
   _gst_dp_initialized = TRUE;
 
+  gst_dp_version_get_type ();
+
   GST_DEBUG_CATEGORY_INIT (data_protocol_debug, "gdp", 0,
       "GStreamer Data Protocol");
 }
 
-/*** PUBLIC FUNCTIONS ***/
-
 /**
  * gst_dp_header_payload_length:
  * @header: the byte header of the packet array
@@ -148,6 +333,8 @@ gst_dp_header_payload_type (const guint8 * header)
   return GST_DP_HEADER_PAYLOAD_TYPE (header);
 }
 
+/*** PACKETIZER FUNCTIONS ***/
+
 /**
  * gst_dp_header_from_buffer:
  * @buffer: a #GstBuffer to create a header for
@@ -157,66 +344,24 @@ gst_dp_header_payload_type (const guint8 * header)
  *
  * Creates a GDP header from the given buffer.
  *
+ * Deprecated: use a #GstDPPacketizer
+ *
  * Returns: %TRUE if the header was successfully created.
  */
-
 gboolean
 gst_dp_header_from_buffer (const GstBuffer * buffer, GstDPHeaderFlag flags,
     guint * length, guint8 ** header)
 {
-  guint8 *h;
-  guint16 crc;
-  guint16 flags_mask;
+  return gst_dp_header_from_buffer_any (buffer, flags, length, header,
+      GST_DP_VERSION_0_2);
+}
 
-  g_return_val_if_fail (GST_IS_BUFFER (buffer), FALSE);
-  g_return_val_if_fail (header, FALSE);
-
-  *length = GST_DP_HEADER_LENGTH;
-  h = g_malloc0 (GST_DP_HEADER_LENGTH);
-
-  /* version, flags, type */
-  h[0] = (guint8) GST_DP_VERSION_MAJOR;
-  h[1] = (guint8) GST_DP_VERSION_MINOR;
-  h[2] = (guint8) flags;
-  h[3] = 0;                     /* padding byte */
-  GST_WRITE_UINT16_BE (h + 4, GST_DP_PAYLOAD_BUFFER);
-
-  /* buffer properties */
-  GST_WRITE_UINT32_BE (h + 6, GST_BUFFER_SIZE (buffer));
-  GST_WRITE_UINT64_BE (h + 10, GST_BUFFER_TIMESTAMP (buffer));
-  GST_WRITE_UINT64_BE (h + 18, GST_BUFFER_DURATION (buffer));
-  GST_WRITE_UINT64_BE (h + 26, GST_BUFFER_OFFSET (buffer));
-  GST_WRITE_UINT64_BE (h + 34, GST_BUFFER_OFFSET_END (buffer));
-
-  /* data flags */
-  /* we only copy KEY_UNIT,DELTA_UNIT and IN_CAPS flags */
-  flags_mask = GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_IN_CAPS |
-      GST_BUFFER_FLAG_DELTA_UNIT;
-
-  GST_WRITE_UINT16_BE (h + 42, GST_BUFFER_FLAGS (buffer) & flags_mask);
-
-  /* ABI padding */
-  GST_WRITE_UINT32_BE (h + 44, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 48, (guint64) 0);
-
-  /* CRC */
-  crc = 0;
-  if (flags & GST_DP_HEADER_FLAG_CRC_HEADER) {
-    /* we don't crc the last four bytes of the header since they are crc's */
-    crc = gst_dp_crc (h, 58);
-  }
-  GST_WRITE_UINT16_BE (h + 58, crc);
-
-  crc = 0;
-  if (flags & GST_DP_HEADER_FLAG_CRC_PAYLOAD) {
-    crc = gst_dp_crc (GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
-  }
-  GST_WRITE_UINT16_BE (h + 60, crc);
-
-  GST_LOG ("created header from buffer:");
-  gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
-  *header = h;
-  return TRUE;
+static gboolean
+gst_dp_header_from_buffer_1_0 (const GstBuffer * buffer, GstDPHeaderFlag flags,
+    guint * length, guint8 ** header)
+{
+  return gst_dp_header_from_buffer_any (buffer, flags, length, header,
+      GST_DP_VERSION_1_0);
 }
 
  /**
@@ -229,63 +374,24 @@ gst_dp_header_from_buffer (const GstBuffer * buffer, GstDPHeaderFlag flags,
  *
  * Creates a GDP packet from the given caps.
  *
+ * Deprecated: use a #GstDPPacketizer
+ *
  * Returns: %TRUE if the packet was successfully created.
  */
 gboolean
 gst_dp_packet_from_caps (const GstCaps * caps, GstDPHeaderFlag flags,
     guint * length, guint8 ** header, guint8 ** payload)
 {
-  guint8 *h;
-  guint16 crc;
-  guchar *string;
+  return gst_dp_packet_from_caps_any (caps, flags, length, header, payload,
+      GST_DP_VERSION_0_2);
+}
 
-  /* FIXME: GST_IS_CAPS doesn't work
-     g_return_val_if_fail (GST_IS_CAPS (caps), FALSE); */
-  g_return_val_if_fail (caps, FALSE);
-  g_return_val_if_fail (header, FALSE);
-  g_return_val_if_fail (payload, FALSE);
-
-  *length = GST_DP_HEADER_LENGTH;
-  h = g_malloc0 (GST_DP_HEADER_LENGTH);
-
-  string = (guchar *) gst_caps_to_string (caps);
-
-  /* version, flags, type */
-  h[0] = (guint8) GST_DP_VERSION_MAJOR;
-  h[1] = (guint8) GST_DP_VERSION_MINOR;
-  h[2] = (guint8) flags;
-  h[3] = 0;                     /* padding bytes */
-  GST_WRITE_UINT16_BE (h + 4, GST_DP_PAYLOAD_CAPS);
-
-  /* buffer properties */
-  GST_WRITE_UINT32_BE (h + 6, strlen ((gchar *) string) + 1);   /* include trailing 0 */
-  GST_WRITE_UINT64_BE (h + 10, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 18, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 26, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 34, (guint64) 0);
-
-  /* ABI padding */
-  GST_WRITE_UINT64_BE (h + 42, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 50, (guint64) 0);
-
-  /* CRC */
-  crc = 0;
-  if (flags & GST_DP_HEADER_FLAG_CRC_HEADER) {
-    crc = gst_dp_crc (h, 58);
-  }
-  GST_WRITE_UINT16_BE (h + 58, crc);
-
-  crc = 0;
-  if (flags & GST_DP_HEADER_FLAG_CRC_PAYLOAD) {
-    crc = gst_dp_crc (string, strlen ((gchar *) string) + 1);
-  }
-  GST_WRITE_UINT16_BE (h + 60, crc);
-
-  GST_LOG ("created header from caps:");
-  gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
-  *header = h;
-  *payload = string;
-  return TRUE;
+gboolean
+gst_dp_packet_from_caps_1_0 (const GstCaps * caps, GstDPHeaderFlag flags,
+    guint * length, guint8 ** header, guint8 ** payload)
+{
+  return gst_dp_packet_from_caps_any (caps, flags, length, header, payload,
+      GST_DP_VERSION_1_0);
 }
 
 /**
@@ -298,6 +404,8 @@ gst_dp_packet_from_caps (const GstCaps * caps, GstDPHeaderFlag flags,
  *
  * Creates a GDP packet from the given event.
  *
+ * Deprecated: use a #GstDPPacketizer
+ *
  * Returns: %TRUE if the packet was successfully created.
  */
 gboolean
@@ -305,7 +413,6 @@ gst_dp_packet_from_event (const GstEvent * event, GstDPHeaderFlag flags,
     guint * length, guint8 ** header, guint8 ** payload)
 {
   guint8 *h;
-  guint16 crc;
   guint pl_length;              /* length of payload */
 
   g_return_val_if_fail (event, FALSE);
@@ -319,7 +426,7 @@ gst_dp_packet_from_event (const GstEvent * event, GstDPHeaderFlag flags,
   /* first construct payload, since we need the length */
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_UNKNOWN:
-      g_warning ("Unknown event, ignoring");
+      GST_WARNING ("Unknown event, ignoring");
       *length = 0;
       g_free (h);
       return FALSE;
@@ -355,21 +462,19 @@ gst_dp_packet_from_event (const GstEvent * event, GstDPHeaderFlag flags,
     case GST_EVENT_QOS:
     case GST_EVENT_NAVIGATION:
     case GST_EVENT_TAG:
-      g_warning ("Unhandled event type %d, ignoring", GST_EVENT_TYPE (event));
+      GST_WARNING ("Unhandled event type %d, ignoring", GST_EVENT_TYPE (event));
+      *length = 0;
+      g_free (h);
       return FALSE;
     default:
-      g_warning ("Unknown event type %d, ignoring", GST_EVENT_TYPE (event));
+      GST_WARNING ("Unknown event type %d, ignoring", GST_EVENT_TYPE (event));
       *length = 0;
       g_free (h);
       return FALSE;
   }
 
   /* version, flags, type */
-  h[0] = (guint8) GST_DP_VERSION_MAJOR;
-  h[1] = (guint8) GST_DP_VERSION_MINOR;
-  h[2] = (guint8) flags;
-  h[3] = 0;                     /* padding byte */
-  GST_WRITE_UINT16_BE (h + 4,
+  GST_DP_INIT_HEADER (h, GST_DP_VERSION_0_2, flags,
       GST_DP_PAYLOAD_EVENT_NONE + GST_EVENT_TYPE (event));
 
   /* length */
@@ -377,23 +482,7 @@ gst_dp_packet_from_event (const GstEvent * event, GstDPHeaderFlag flags,
   /* timestamp */
   GST_WRITE_UINT64_BE (h + 10, GST_EVENT_TIMESTAMP (event));
 
-  /* ABI padding */
-  GST_WRITE_UINT64_BE (h + 42, (guint64) 0);
-  GST_WRITE_UINT64_BE (h + 50, (guint64) 0);
-
-  /* CRC */
-  crc = 0;
-  if (flags & GST_DP_HEADER_FLAG_CRC_HEADER) {
-    crc = gst_dp_crc (h, 58);
-  }
-  GST_WRITE_UINT16_BE (h + 58, crc);
-
-  crc = 0;
-  /* events can have a NULL payload */
-  if (*payload && flags & GST_DP_HEADER_FLAG_CRC_PAYLOAD) {
-    crc = gst_dp_crc (*payload, strlen ((gchar *) * payload) + 1);
-  }
-  GST_WRITE_UINT16_BE (h + 60, crc);
+  GST_DP_SET_CRC (h, flags, *payload, pl_length);
 
   GST_LOG ("created header from event:");
   gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
@@ -401,6 +490,50 @@ gst_dp_packet_from_event (const GstEvent * event, GstDPHeaderFlag flags,
   return TRUE;
 }
 
+static gboolean
+gst_dp_packet_from_event_1_0 (const GstEvent * event, GstDPHeaderFlag flags,
+    guint * length, guint8 ** header, guint8 ** payload)
+{
+  guint8 *h;
+  guint32 pl_length;            /* length of payload */
+  guchar *string = NULL;
+
+  g_return_val_if_fail (event, FALSE);
+  g_return_val_if_fail (GST_IS_EVENT (event), FALSE);
+  g_return_val_if_fail (header, FALSE);
+  g_return_val_if_fail (payload, FALSE);
+
+  *length = GST_DP_HEADER_LENGTH;
+  h = g_malloc0 (GST_DP_HEADER_LENGTH);
+
+  if (event->structure) {
+    string = (guchar *) gst_structure_to_string (event->structure);
+    GST_LOG ("event %p has structure, string %s", event, string);
+    pl_length = strlen ((gchar *) string) + 1;  /* include trailing 0 */
+  } else {
+    GST_LOG ("event %p has no structure");
+    pl_length = 0;
+  }
+
+  /* version, flags, type */
+  GST_DP_INIT_HEADER (h, GST_DP_VERSION_1_0, flags,
+      GST_DP_PAYLOAD_EVENT_NONE + GST_EVENT_TYPE (event));
+
+  /* length */
+  GST_WRITE_UINT32_BE (h + 6, pl_length);
+  /* timestamp */
+  GST_WRITE_UINT64_BE (h + 10, GST_EVENT_TIMESTAMP (event));
+
+  GST_DP_SET_CRC (h, flags, *payload, pl_length);
+
+  GST_LOG ("created header from event:");
+  gst_dp_dump_byte_array (h, GST_DP_HEADER_LENGTH);
+  *header = h;
+  *payload = string;
+  return TRUE;
+}
+
+/*** DEPACKETIZING FUNCTIONS ***/
 
 /**
  * gst_dp_buffer_from_header:
@@ -462,31 +595,17 @@ gst_dp_caps_from_packet (guint header_length, const guint8 * header,
   return caps;
 }
 
-/**
- * gst_dp_event_from_packet:
- * @header_length: the length of the packet header
- * @header: the byte array of the packet header
- * @payload: the byte array of the packet payload
- *
- * Creates a newly allocated #GstEvent from the given packet.
- *
- * Returns: A #GstEvent if the event was successfully created, 
- *          or NULL if an event could not be read from the payload.
- */
-GstEvent *
-gst_dp_event_from_packet (guint header_length, const guint8 * header,
+static GstEvent *
+gst_dp_event_from_packet_0_2 (guint header_length, const guint8 * header,
     const guint8 * payload)
 {
   GstEvent *event = NULL;
   GstEventType type;
 
-  g_return_val_if_fail (header, NULL);
-  /* payload can be NULL, e.g. for an EOS event */
-
   type = GST_DP_HEADER_PAYLOAD_TYPE (header) - GST_DP_PAYLOAD_EVENT_NONE;
   switch (type) {
     case GST_EVENT_UNKNOWN:
-      g_warning ("Unknown event, ignoring");
+      GST_WARNING ("Unknown event, ignoring");
       return FALSE;
     case GST_EVENT_EOS:
     case GST_EVENT_FLUSH_START:
@@ -520,16 +639,68 @@ gst_dp_event_from_packet (guint header_length, const guint8 * header,
     case GST_EVENT_QOS:
     case GST_EVENT_NAVIGATION:
     case GST_EVENT_TAG:
-      g_warning ("Unhandled event type %d, ignoring", GST_EVENT_TYPE (event));
+      GST_WARNING ("Unhandled event type %d, ignoring", type);
       return FALSE;
     default:
-      g_warning ("Unknown event type %d, ignoring", GST_EVENT_TYPE (event));
+      GST_WARNING ("Unknown event type %d, ignoring", type);
       return FALSE;
   }
 
   return event;
 }
 
+static GstEvent *
+gst_dp_event_from_packet_1_0 (guint header_length, const guint8 * header,
+    const guint8 * payload)
+{
+  GstEvent *event = NULL;
+  GstEventType type;
+  gchar *string;
+  GstStructure *s;
+
+  type = GST_DP_HEADER_PAYLOAD_TYPE (header) - GST_DP_PAYLOAD_EVENT_NONE;
+  string = g_strndup ((gchar *) payload, GST_DP_HEADER_PAYLOAD_LENGTH (header));
+  s = gst_structure_from_string (string, NULL);
+  g_free (string);
+  if (!s)
+    return NULL;
+  event = gst_event_new_custom (type, s);
+  return event;
+}
+
+
+/**
+ * gst_dp_event_from_packet:
+ * @header_length: the length of the packet header
+ * @header: the byte array of the packet header
+ * @payload: the byte array of the packet payload
+ *
+ * Creates a newly allocated #GstEvent from the given packet.
+ *
+ * Returns: A #GstEvent if the event was successfully created,
+ *          or NULL if an event could not be read from the payload.
+ */
+GstEvent *
+gst_dp_event_from_packet (guint header_length, const guint8 * header,
+    const guint8 * payload)
+{
+  guint8 major, minor;
+
+  g_return_val_if_fail (header, NULL);
+
+  major = GST_DP_HEADER_MAJOR_VERSION (header);
+  minor = GST_DP_HEADER_MINOR_VERSION (header);
+
+  if (major == 0 && minor == 2)
+    return gst_dp_event_from_packet_0_2 (header_length, header, payload);
+  else if (major == 1 && minor == 0)
+    return gst_dp_event_from_packet_1_0 (header_length, header, payload);
+  else {
+    GST_ERROR ("Unknown GDP version %d.%d", major, minor);
+    return NULL;
+  }
+}
+
 /**
  * gst_dp_validate_header:
  * @header_length: the length of the packet header
@@ -547,7 +718,7 @@ gst_dp_validate_header (guint header_length, const guint8 * header)
   if (!(GST_DP_HEADER_FLAGS (header) & GST_DP_HEADER_FLAG_CRC_HEADER))
     return TRUE;
   crc_read = GST_DP_HEADER_CRC_HEADER (header);
-  /* don't included the last two crc fields for the crc check */
+  /* don't include the last two crc fields for the crc check */
   crc_calculated = gst_dp_crc (header, header_length - 4);
   if (crc_read != crc_calculated) {
     GST_WARNING ("header crc mismatch: read %02x, calculated %02x", crc_read,
@@ -609,3 +780,51 @@ gst_dp_validate_packet (guint header_length, const guint8 * header,
 
   return TRUE;
 }
+
+/**
+ * gst_dp_packetizer_new:
+ * @version: the #GstDPVersion of the protocol to packetize for.
+ *
+ * Creates a new packetizer.
+ *
+ * Returns: a newly allocated #GstDPPacketizer
+ */
+GstDPPacketizer *
+gst_dp_packetizer_new (GstDPVersion version)
+{
+  GstDPPacketizer *ret;
+
+  ret = g_malloc0 (sizeof (GstDPPacketizer));
+  ret->version = version;
+
+  switch (version) {
+    case GST_DP_VERSION_0_2:
+      ret->header_from_buffer = gst_dp_header_from_buffer;
+      ret->packet_from_caps = gst_dp_packet_from_caps;
+      ret->packet_from_event = gst_dp_packet_from_event;
+      break;
+    case GST_DP_VERSION_1_0:
+      ret->header_from_buffer = gst_dp_header_from_buffer_1_0;
+      ret->packet_from_caps = gst_dp_packet_from_caps_1_0;
+      ret->packet_from_event = gst_dp_packet_from_event_1_0;
+      break;
+    default:
+      g_free (ret);
+      ret = NULL;
+      break;
+  }
+
+  return ret;
+}
+
+/**
+ * gst_dp_packetizer_free:
+ * @packetizer: the #GstDPPacketizer to free.
+ *
+ * Free the given packetizer.
+ */
+void
+gst_dp_packetizer_free (GstDPPacketizer * packetizer)
+{
+  g_free (packetizer);
+}
