@@ -29,7 +29,7 @@
 #include <gst/base/gstbasetransform.h>
 
 
-static GstElementDetails gst_capsfilter_details =
+static const GstElementDetails gst_capsfilter_details =
 GST_ELEMENT_DETAILS ("CapsFilter",
     "Generic",
     "Pass data without modification, limiting formats",
@@ -124,17 +124,17 @@ gst_capsfilter_class_init (GstCapsFilterClass * klass)
   GObjectClass *gobject_class;
   GstBaseTransformClass *trans_class;
 
-  gobject_class = (GObjectClass *) klass;
+  gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->set_property = gst_capsfilter_set_property;
   gobject_class->get_property = gst_capsfilter_get_property;
   gobject_class->dispose = gst_capsfilter_dispose;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FILTER_CAPS,
+  g_object_class_install_property (gobject_class, PROP_FILTER_CAPS,
       g_param_spec_boxed ("caps", _("Filter caps"),
           _("Restrict the possible allowed capabilities (NULL means ANY)"),
           GST_TYPE_CAPS, G_PARAM_READWRITE));
 
-  trans_class = (GstBaseTransformClass *) klass;
+  trans_class = GST_BASE_TRANSFORM_CLASS (klass);
   trans_class->transform_caps = gst_capsfilter_transform_caps;
   trans_class->transform_ip = gst_capsfilter_transform_ip;
   trans_class->prepare_output_buffer = gst_capsfilter_prepare_buf;
@@ -257,6 +257,9 @@ gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
       g_return_val_if_fail (out_caps != NULL, GST_FLOW_ERROR);
     }
 
+    out_caps = gst_caps_make_writable (out_caps);
+    gst_caps_do_simplify (out_caps);
+
     if (gst_caps_is_fixed (out_caps) && !gst_caps_is_empty (out_caps)) {
       GST_DEBUG_OBJECT (trans, "Have fixed output caps %"
           GST_PTR_FORMAT " to apply to buffer with no caps", out_caps);
@@ -272,6 +275,8 @@ gst_capsfilter_prepare_buf (GstBaseTransform * trans, GstBuffer * input,
       if (GST_PAD_CAPS (trans->srcpad) == NULL)
         gst_pad_set_caps (trans->srcpad, out_caps);
     } else {
+      GST_DEBUG_OBJECT (trans, "Have unfixed output caps %" GST_PTR_FORMAT,
+          out_caps);
       gst_caps_unref (out_caps);
     }
   }
