@@ -52,7 +52,7 @@ static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
 GST_DEBUG_CATEGORY_STATIC (gst_file_sink_debug);
 #define GST_CAT_DEFAULT gst_file_sink_debug
 
-static GstElementDetails gst_file_sink_details =
+static const GstElementDetails gst_file_sink_details =
 GST_ELEMENT_DETAILS ("File Sink",
     "Sink/File",
     "Write stream to a file",
@@ -123,7 +123,7 @@ gst_file_sink_class_init (GstFileSinkClass * klass)
   gobject_class->set_property = gst_file_sink_set_property;
   gobject_class->get_property = gst_file_sink_get_property;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LOCATION,
+  g_object_class_install_property (gobject_class, ARG_LOCATION,
       g_param_spec_string ("location", "File Location",
           "Location of the file to write", NULL, G_PARAM_READWRITE));
 
@@ -412,20 +412,30 @@ gst_file_sink_render (GstBaseSink * sink, GstBuffer * buffer)
   GST_DEBUG_OBJECT (filesink, "writing %u bytes at %" G_GUINT64_FORMAT,
       size, cur_pos);
 
-  if (fwrite (GST_BUFFER_DATA (buffer), size, 1, filesink->file) != 1)
-    goto handle_error;
+  if (size > 0 && GST_BUFFER_DATA (buffer) != NULL) {
+    if (fwrite (GST_BUFFER_DATA (buffer), size, 1, filesink->file) != 1)
+      goto handle_error;
 
-  filesink->data_written += size - back_pending;
+    filesink->data_written += size - back_pending;
+  }
 
   return GST_FLOW_OK;
 
 handle_error:
-
-  GST_ELEMENT_ERROR (filesink, RESOURCE, WRITE,
-      (_("Error while writing to file \"%s\"."), filesink->filename),
-      ("%s", g_strerror (errno)));
-
-  return GST_FLOW_ERROR;
+  {
+    switch (errno) {
+      case ENOSPC:{
+        GST_ELEMENT_ERROR (filesink, RESOURCE, NO_SPACE_LEFT, (NULL), (NULL));
+        break;
+      }
+      default:{
+        GST_ELEMENT_ERROR (filesink, RESOURCE, WRITE,
+            (_("Error while writing to file \"%s\"."), filesink->filename),
+            ("%s", g_strerror (errno)));
+      }
+    }
+    return GST_FLOW_ERROR;
+  }
 }
 
 static gboolean
