@@ -57,10 +57,13 @@ GST_START_TEST (test_interface)
   gst_bin_add (bin, filesrc);
 
   fail_unless (gst_bin_get_by_interface (bin, GST_TYPE_URI_HANDLER) == filesrc);
+  gst_object_unref (filesrc);
+
   it = gst_bin_iterate_all_by_interface (bin, GST_TYPE_URI_HANDLER);
   fail_unless (it != NULL);
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_OK);
   fail_unless (item == (gpointer) filesrc);
+  gst_object_unref (GST_OBJECT (item));
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_DONE);
   gst_iterator_free (it);
 
@@ -68,11 +71,11 @@ GST_START_TEST (test_interface)
       gst_element_factory_make ("identity", NULL),
       gst_element_factory_make ("identity", NULL),
       gst_element_factory_make ("identity", NULL), NULL);
-  fail_unless (gst_bin_get_by_interface (bin, GST_TYPE_URI_HANDLER) == filesrc);
   it = gst_bin_iterate_all_by_interface (bin, GST_TYPE_URI_HANDLER);
   fail_unless (it != NULL);
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_OK);
   fail_unless (item == (gpointer) filesrc);
+  gst_object_unref (GST_OBJECT (item));
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_DONE);
   gst_iterator_free (it);
 
@@ -83,10 +86,10 @@ GST_START_TEST (test_interface)
       gst_element_factory_make ("identity", NULL),
       gst_element_factory_make ("identity", NULL),
       GST_ELEMENT (bin2), gst_element_factory_make ("identity", NULL), NULL);
-  fail_unless (gst_bin_get_by_interface (bin, GST_TYPE_URI_HANDLER) == filesrc);
   it = gst_bin_iterate_all_by_interface (bin, GST_TYPE_URI_HANDLER);
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_OK);
   fail_unless (item == (gpointer) filesrc);
+  gst_object_unref (GST_OBJECT (item));
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_DONE);
   gst_iterator_free (it);
 
@@ -94,8 +97,11 @@ GST_START_TEST (test_interface)
   gst_bin_add (bin2, gst_element_factory_make ("filesrc", NULL));
   it = gst_bin_iterate_all_by_interface (bin, GST_TYPE_URI_HANDLER);
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_OK);
+  gst_object_unref (GST_OBJECT (item));
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_OK);
+  gst_object_unref (GST_OBJECT (item));
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_OK);
+  gst_object_unref (GST_OBJECT (item));
   fail_unless (gst_iterator_next (it, &item) == GST_ITERATOR_DONE);
   gst_iterator_free (it);
 
@@ -562,6 +568,9 @@ GST_START_TEST (test_children_state_change_order_flagged_sink)
   pop_messages (bus, 4);        /* pop playing => paused messages off the bus */
   pop_messages (bus, 4);        /* pop paused => ready messages off the bus */
 
+  while (GST_OBJECT_REFCOUNT_VALUE (pipeline) > 1)
+    THREAD_SWITCH ();
+
   ASSERT_OBJECT_REFCOUNT (src, "src", 1);
   ASSERT_OBJECT_REFCOUNT (sink, "sink", 1);
   ASSERT_OBJECT_REFCOUNT (pipeline, "pipeline", 1);
