@@ -25,14 +25,13 @@
 #  include "config.h"
 #endif
 
-#include <gst/gst.h>
 #include <gst/controller/gstcontroller.h>
 
-#include "gst/gst-i18n-app.h"
+#include "tools.h"
 
-#include <stdlib.h>
 #include <string.h>
 #include <locale.h>
+#include <libintl.h>
 #include <glib/gprintf.h>
 
 static char *_name;
@@ -248,7 +247,8 @@ print_interfaces (GType type)
 
   if (ifaces) {
     if (n_ifaces) {
-      g_print ("%sImplemented Interfaces:\n", _name);
+      g_print ("%s", _name);
+      g_print (_("Implemented Interfaces:\n"));
       iface = ifaces;
       while (*iface) {
         g_print ("%s  %s\n", _name, g_type_name (*iface));
@@ -289,15 +289,18 @@ print_element_properties_info (GstElement * element)
     if (param->flags & G_PARAM_READABLE) {
       g_object_get_property (G_OBJECT (element), param->name, &value);
       readable = TRUE;
-      g_print ((first_flag ? "readable" : ", readable"));
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("readable"));
       first_flag = FALSE;
     }
     if (param->flags & G_PARAM_WRITABLE) {
-      g_print ((first_flag ? "writable" : ", writable"));
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("writable"));
       first_flag = FALSE;
     }
     if (param->flags & GST_PARAM_CONTROLLABLE) {
-      g_print ((first_flag ? "controllable" : ", controllable"));
+      g_print ((first_flag ? "" : ", "));
+      g_print (_("controllable"));
       first_flag = FALSE;
     }
     n_print ("\n");
@@ -916,8 +919,12 @@ print_element_list (gboolean print_all)
 
   gst_plugin_list_free (orig_plugins);
 
-  g_print ("\nTotal plugins: %d\nTotal features: %d\n",
-      plugincount, featurecount);
+  g_print ("\n");
+  g_print (_("Total count: "));
+  g_print (ngettext ("%d plugin", "%d plugins", plugincount), plugincount);
+  g_print (", ");
+  g_print (ngettext ("%d feature", "%d features", featurecount), featurecount);
+  g_print ("\n");
 }
 
 static void
@@ -1098,6 +1105,7 @@ main (int argc, char *argv[])
   GOptionEntry options[] = {
     {"print-all", 'a', 0, G_OPTION_ARG_NONE, &print_all,
         N_("Print all elements"), NULL},
+    GST_TOOLS_GOPTION_VERSION,
     {NULL}
   };
   GOptionContext *ctx;
@@ -1109,7 +1117,7 @@ main (int argc, char *argv[])
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  ctx = g_option_context_new ("gst-inspect");
+  ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
   if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
@@ -1118,6 +1126,8 @@ main (int argc, char *argv[])
   }
   g_option_context_free (ctx);
 
+  gst_tools_print_version ("gst-inspect");
+
   if (print_all && argc > 2) {
     g_print ("-a requires no extra arguments\n");
     return 1;
@@ -1160,12 +1170,12 @@ main (int argc, char *argv[])
             print_plugin_info (plugin);
             print_plugin_features (plugin);
           } else {
-            g_print ("Error loading plugin file: %s\n", error->message);
+            g_print (_("Could not load plugin file: %s\n"), error->message);
             g_error_free (error);
             return -1;
           }
         } else {
-          g_print ("No such element or plugin '%s'\n", arg);
+          g_print (_("No such element or plugin '%s'\n"), arg);
           return -1;
         }
       }
