@@ -1,10 +1,13 @@
-#include "tools.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
-#include <gst/control/control.h>
 #include <string.h>
 #include <locale.h>
 #include <glib/gprintf.h>
 
+#include "tools.h"
+
 #define PUT_START_TAG(pfx,tag)                                  \
 G_STMT_START{                                                   \
   g_print ("%*.*s<%s>\n", pfx, pfx, "", tag);                   \
@@ -38,7 +41,7 @@ G_STMT_START{                                                   \
 
 #define PUT_STRING(pfx, str, a...)                              \
 G_STMT_START{                                                   \
-  g_print ("%*.*s"str"\n", pfx, pfx, "" , ##a);	   		\
+  g_print ("%*.*s"str"\n", pfx, pfx, "" , ##a);                 \
 }G_STMT_END
 
 #else
@@ -70,6 +73,7 @@ print_caps (const GstCaps * caps, gint pfx)
   g_free (s);
 }
 
+#if 0
 static void
 print_formats (const GstFormat * formats, gint pfx)
 {
@@ -86,6 +90,7 @@ print_formats (const GstFormat * formats, gint pfx)
     formats++;
   }
 }
+#endif
 
 static void
 print_query_types (const GstQueryType * types, gint pfx)
@@ -104,6 +109,7 @@ print_query_types (const GstQueryType * types, gint pfx)
   }
 }
 
+#if 0
 static void
 print_event_masks (const GstEventMask * masks, gint pfx)
 {
@@ -153,6 +159,7 @@ print_event_masks (const GstEventMask * masks, gint pfx)
   }
 #endif
 }
+#endif
 
 static void
 output_hierarchy (GType type, gint level, gint * maxlevel)
@@ -176,7 +183,8 @@ static void
 print_element_properties (GstElement * element, gint pfx)
 {
   GParamSpec **property_specs;
-  guint num_properties, i;
+  guint num_properties;
+  gint i;
   gboolean readable;
 
   property_specs = g_object_class_list_properties
@@ -349,58 +357,6 @@ print_element_properties (GstElement * element, gint pfx)
   g_free (property_specs);
 }
 
-static void
-print_element_dynparamaters (GstElement * element, gint pfx)
-{
-  GstDParamManager *dpman;
-  GParamSpec **specs;
-  gint x;
-
-  PUT_START_TAG (pfx, "dyn-params");
-
-  if ((dpman = gst_dpman_get_manager (element))) {
-    specs = gst_dpman_list_dparam_specs (dpman);
-    for (x = 0; specs[x] != NULL; x++) {
-      PUT_START_TAG (pfx + 1, "dyn-param");
-
-      PUT_ESCAPED (pfx + 2, "name", g_param_spec_get_name (specs[x]));
-      PUT_ESCAPED (pfx + 2, "type", g_type_name (specs[x]->value_type));
-      PUT_ESCAPED (pfx + 2, "nick", g_param_spec_get_nick (specs[x]));
-      PUT_ESCAPED (pfx + 2, "blurb", g_param_spec_get_blurb (specs[x]));
-
-      switch (G_PARAM_SPEC_VALUE_TYPE (specs[x])) {
-        case G_TYPE_INT64:
-          PUT_STRING (pfx + 2,
-              "<range min=\"%" G_GINT64_FORMAT "\" max=\"%" G_GINT64_FORMAT
-              "\"/>", ((GParamSpecInt64 *) specs[x])->minimum,
-              ((GParamSpecInt64 *) specs[x])->maximum);
-          PUT_STRING (pfx + 2, "<default>%" G_GINT64_FORMAT "</default>",
-              ((GParamSpecInt64 *) specs[x])->default_value);
-          break;
-        case G_TYPE_INT:
-          PUT_STRING (pfx + 2, "<range min=\"%d\" max=\"%d\"/>",
-              ((GParamSpecInt *) specs[x])->minimum,
-              ((GParamSpecInt *) specs[x])->maximum);
-          PUT_STRING (pfx + 2, "<default>%d</default>",
-              ((GParamSpecInt *) specs[x])->default_value);
-          break;
-        case G_TYPE_FLOAT:
-          PUT_STRING (pfx + 2, "<range min=\"%f\" max=\"%f\"/>",
-              ((GParamSpecFloat *) specs[x])->minimum,
-              ((GParamSpecFloat *) specs[x])->maximum);
-          PUT_STRING (pfx + 2, "<default>%f</default>",
-              ((GParamSpecFloat *) specs[x])->default_value);
-          break;
-        default:
-          break;
-      }
-      PUT_END_TAG (pfx + 1, "dyn-param");
-    }
-    g_free (specs);
-  }
-  PUT_END_TAG (pfx, "dyn-params");
-}
-
 static void
 print_element_signals (GstElement * element, gint pfx)
 {
@@ -468,10 +424,7 @@ print_element_info (GstElementFactory * factory)
   GstElementClass *gstelement_class;
   GList *pads;
   GstPad *pad;
-  GstRealPad *realpad;
-  GstPadTemplate *padtemplate;
-  GList *children;
-  GstElement *child;
+  GstStaticPadTemplate *padtemplate;
   gint maxlevel = 0;
 
   element = gst_element_factory_create (factory, "element");
@@ -496,9 +449,9 @@ print_element_info (GstElementFactory * factory)
 
   PUT_START_TAG (1, "pad-templates");
   if (factory->numpadtemplates) {
-    pads = factory->padtemplates;
+    pads = factory->staticpadtemplates;
     while (pads) {
-      padtemplate = (GstPadTemplate *) (pads->data);
+      padtemplate = (GstStaticPadTemplate *) (pads->data);
       pads = g_list_next (pads);
 
       PUT_START_TAG (2, "pad-template");
@@ -522,8 +475,8 @@ print_element_info (GstElementFactory * factory)
       } else
         PUT_ESCAPED (3, "presence", "unknown");
 
-      if (padtemplate->caps) {
-        print_caps (padtemplate->caps, 3);
+      if (padtemplate->static_caps.string) {
+        print_caps (gst_static_caps_get (&padtemplate->static_caps), 3);
       }
       PUT_END_TAG (2, "pad-template");
     }
@@ -531,40 +484,16 @@ print_element_info (GstElementFactory * factory)
   PUT_END_TAG (1, "pad-templates");
 
   PUT_START_TAG (1, "element-flags");
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_COMPLEX)) {
-    PUT_ESCAPED (2, "flag", "GST_ELEMENT_COMPLEX");
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_DECOUPLED)) {
-    PUT_ESCAPED (2, "flag", "GST_ELEMENT_DECOUPLED");
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_THREAD_SUGGESTED)) {
-    PUT_ESCAPED (2, "flag", "GST_ELEMENT_THREADSUGGESTED");
-  }
-  if (GST_FLAG_IS_SET (element, GST_ELEMENT_EVENT_AWARE)) {
-    PUT_ESCAPED (2, "flag", "GST_ELEMENT_EVENT_AWARE");
-  }
   PUT_END_TAG (1, "element-flags");
 
   if (GST_IS_BIN (element)) {
     PUT_START_TAG (1, "bin-flags");
 
-    if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_MANAGER)) {
-      PUT_ESCAPED (2, "flag", "GST_BIN_FLAG_MANAGER");
-    }
-    if (GST_FLAG_IS_SET (element, GST_BIN_SELF_SCHEDULABLE)) {
-      PUT_ESCAPED (2, "flag", "GST_BIN_SELF_SCHEDULABLE");
-    }
-    if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_PREFER_COTHREADS)) {
-      PUT_ESCAPED (2, "flag", "GST_BIN_FLAG_PREFER_COTHREADS");
-    }
     PUT_END_TAG (1, "bin-flags");
   }
 
 
   PUT_START_TAG (1, "element-implementation");
-  if (element->loopfunc)
-    PUT_STRING (2, "<loop-based function=\"%s\"/>",
-        GST_DEBUG_FUNCPTR_NAME (element->loopfunc));
 
   PUT_STRING (2, "<state-change function=\"%s\"/>",
       GST_DEBUG_FUNCPTR_NAME (gstelement_class->change_state));
@@ -600,11 +529,10 @@ print_element_info (GstElementFactory * factory)
   if (element->numpads) {
     const GList *pads;
 
-    pads = gst_element_get_pad_list (element);
+    pads = element->pads;
     while (pads) {
       pad = GST_PAD (pads->data);
       pads = g_list_next (pads);
-      realpad = GST_PAD_REALIZE (pad);
 
       PUT_START_TAG (2, "pad");
       PUT_ESCAPED (3, "name", gst_pad_get_name (pad));
@@ -616,58 +544,40 @@ print_element_info (GstElementFactory * factory)
       else
         PUT_ESCAPED (3, "direction", "unknown");
 
-      if (GST_IS_GHOST_PAD (pad))
-        PUT_ESCAPED (3, "ghost", gst_pad_get_name (pad));
-
       if (pad->padtemplate)
         PUT_ESCAPED (3, "template", pad->padtemplate->name_template);
 
       PUT_START_TAG (3, "implementation");
-      if (realpad->chainfunc)
+      if (pad->chainfunc)
         PUT_STRING (4, "<chain-based function=\"%s\"/>",
-            GST_DEBUG_FUNCPTR_NAME (realpad->chainfunc));
-      if (realpad->getfunc)
-        PUT_STRING (4, "<get-based function=\"%s\"/>",
-            GST_DEBUG_FUNCPTR_NAME (realpad->getfunc));
-      if (realpad->formatsfunc != gst_pad_get_formats_default) {
-        PUT_STRING (4, "<formats-function function=\"%s\">",
-            GST_DEBUG_FUNCPTR_NAME (realpad->formatsfunc));
-        print_formats (gst_pad_get_formats (GST_PAD (realpad)), 5);
-        PUT_END_TAG (4, "formats-function");
-      }
-      if (realpad->convertfunc != gst_pad_convert_default)
-        PUT_STRING (4, "<convert-function function=\"%s\"/>",
-            GST_DEBUG_FUNCPTR_NAME (realpad->convertfunc));
-      if (realpad->eventfunc != gst_pad_event_default)
+            GST_DEBUG_FUNCPTR_NAME (pad->chainfunc));
+      if (pad->getrangefunc)
+        PUT_STRING (4, "<get-range-based function=\"%s\"/>",
+            GST_DEBUG_FUNCPTR_NAME (pad->getrangefunc));
+      if (pad->eventfunc != gst_pad_event_default)
         PUT_STRING (4, "<event-function function=\"%s\"/>",
-            GST_DEBUG_FUNCPTR_NAME (realpad->eventfunc));
-      if (realpad->eventmaskfunc != gst_pad_get_event_masks_default) {
-        PUT_STRING (4, "<event-mask-func function=\"%s\">",
-            GST_DEBUG_FUNCPTR_NAME (realpad->eventmaskfunc));
-        print_event_masks (gst_pad_get_event_masks (GST_PAD (realpad)), 5);
-        PUT_END_TAG (4, "event-mask-func");
-      }
-      if (realpad->queryfunc != gst_pad_query_default)
+            GST_DEBUG_FUNCPTR_NAME (pad->eventfunc));
+      if (pad->queryfunc != gst_pad_query_default)
         PUT_STRING (4, "<query-function function=\"%s\"/>",
-            GST_DEBUG_FUNCPTR_NAME (realpad->queryfunc));
-      if (realpad->querytypefunc != gst_pad_get_query_types_default) {
+            GST_DEBUG_FUNCPTR_NAME (pad->queryfunc));
+      if (pad->querytypefunc != gst_pad_get_query_types_default) {
         PUT_STRING (4, "<query-type-func function=\"%s\">",
-            GST_DEBUG_FUNCPTR_NAME (realpad->querytypefunc));
-        print_query_types (gst_pad_get_query_types (GST_PAD (realpad)), 5);
+            GST_DEBUG_FUNCPTR_NAME (pad->querytypefunc));
+        print_query_types (gst_pad_get_query_types (pad), 5);
         PUT_END_TAG (4, "query-type-func");
       }
 
-      if (realpad->intlinkfunc != gst_pad_get_internal_links_default)
+      if (pad->intlinkfunc != gst_pad_get_internal_links_default)
         PUT_STRING (4, "<intlink-function function=\"%s\"/>",
-            GST_DEBUG_FUNCPTR_NAME (realpad->intlinkfunc));
+            GST_DEBUG_FUNCPTR_NAME (pad->intlinkfunc));
 
-      if (realpad->bufferallocfunc)
+      if (pad->bufferallocfunc)
         PUT_STRING (4, "<bufferalloc-function function=\"%s\"/>",
-            GST_DEBUG_FUNCPTR_NAME (realpad->bufferallocfunc));
+            GST_DEBUG_FUNCPTR_NAME (pad->bufferallocfunc));
       PUT_END_TAG (3, "implementation");
 
-      if (realpad->caps) {
-        print_caps (realpad->caps, 3);
+      if (pad->caps) {
+        print_caps (pad->caps, 3);
       }
       PUT_END_TAG (2, "pad");
     }
@@ -675,21 +585,24 @@ print_element_info (GstElementFactory * factory)
   PUT_END_TAG (1, "pads");
 
   print_element_properties (element, 1);
-  print_element_dynparamaters (element, 1);
   print_element_signals (element, 1);
 
   /* for compound elements */
-  if (GST_IS_BIN (element)) {
-    PUT_START_TAG (1, "children");
-    children = (GList *) gst_bin_get_list (GST_BIN (element));
-    while (children) {
-      child = GST_ELEMENT (children->data);
-      children = g_list_next (children);
+  /* FIXME: gst_bin_get_list does not exist anymore
+     if (GST_IS_BIN (element)) {
+     GList *children;
+     GstElement *child;
+     PUT_START_TAG (1, "children");
+     children = (GList *) gst_bin_get_list (GST_BIN (element));
+     while (children) {
+     child = GST_ELEMENT (children->data);
+     children = g_list_next (children);
 
-      PUT_ESCAPED (2, "child", GST_ELEMENT_NAME (child));
-    }
-    PUT_END_TAG (1, "children");
-  }
+     PUT_ESCAPED (2, "child", GST_ELEMENT_NAME (child));
+     }
+     PUT_END_TAG (1, "children");
+     }
+   */
   PUT_END_TAG (0, "element");
 
   return 0;
@@ -700,7 +613,7 @@ print_element_list (void)
 {
   GList *plugins;
 
-  plugins = gst_registry_pool_plugin_list ();
+  plugins = gst_default_registry_get_plugin_list ();
   while (plugins) {
     GList *features;
     GstPlugin *plugin;
@@ -708,7 +621,9 @@ print_element_list (void)
     plugin = (GstPlugin *) (plugins->data);
     plugins = g_list_next (plugins);
 
-    features = gst_plugin_get_feature_list (plugin);
+    features =
+        gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
+        plugin->desc.name);
     while (features) {
       GstPluginFeature *feature;
 
@@ -744,12 +659,6 @@ print_element_list (void)
           }
         } else
           g_print ("%s type: N/A\n", plugin->desc.name);
-      } else if (GST_IS_SCHEDULER_FACTORY (feature)) {
-        GstSchedulerFactory *factory;
-
-        factory = GST_SCHEDULER_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->desc.name,
-            GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       } else {
         g_print ("%s:  %s (%s)\n", plugin->desc.name,
             GST_PLUGIN_FEATURE_NAME (feature),
@@ -769,7 +678,6 @@ print_plugin_info (GstPlugin * plugin)
   gint num_elements = 0;
   gint num_autoplug = 0;
   gint num_types = 0;
-  gint num_schedulers = 0;
   gint num_indexes = 0;
   gint num_other = 0;
 
@@ -783,7 +691,9 @@ print_plugin_info (GstPlugin * plugin)
   g_print ("  Origin URL:\t%s\n", plugin->desc.origin);
   g_print ("\n");
 
-  features = gst_plugin_get_feature_list (plugin);
+  features =
+      gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
+      plugin->desc.name);
 
   while (features) {
     GstPluginFeature *feature;
@@ -822,14 +732,8 @@ print_plugin_info (GstPlugin * plugin)
       } else
         g_print ("%s type: N/A\n", plugin->desc.name);
       num_types++;
-    } else if (GST_IS_SCHEDULER_FACTORY (feature)) {
-      GstSchedulerFactory *factory;
-
-      factory = GST_SCHEDULER_FACTORY (feature);
-      g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
-      num_schedulers++;
     } else {
-      g_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)),
+      g_print ("  %s (%s)\n", GST_OBJECT_NAME (feature),
           g_type_name (G_OBJECT_TYPE (feature)));
       num_other++;
     }
@@ -843,8 +747,6 @@ print_plugin_info (GstPlugin * plugin)
     g_print ("  +-- %d autopluggers\n", num_autoplug);
   if (num_types > 0)
     g_print ("  +-- %d types\n", num_types);
-  if (num_schedulers > 0)
-    g_print ("  +-- %d schedulers\n", num_schedulers);
   if (num_indexes > 0)
     g_print ("  +-- %d indexes\n", num_indexes);
   if (num_other > 0)
@@ -860,21 +762,29 @@ main (int argc, char *argv[])
   GstElementFactory *factory;
   GstPlugin *plugin;
   gchar *so;
-  struct poptOption options[] = {
-    GST_TOOLS_POPT_VERSION,
-    {"gst-inspect-plugin", 'p', POPT_ARG_STRING | POPT_ARGFLAG_STRIP, NULL, 0,
-        N_("Show plugin details"), NULL},
-    {"gst-inspect-scheduler", 's', POPT_ARG_STRING | POPT_ARGFLAG_STRIP, NULL,
-          0,
-        N_("Show scheduler details"), NULL},
-    POPT_TABLEEND
+  GOptionEntry options[] = {
+    {"gst-inspect-plugin", 'p', 0, G_OPTION_ARG_STRING,
+        "Show plugin details", NULL},
+    {"gst-inspect-scheduler", 's', 0, G_OPTION_ARG_STRING,
+        "Show scheduler details", NULL},
+    GST_TOOLS_GOPTION_VERSION,
+    {NULL}
   };
+  GOptionContext *ctx;
+  GError *err = NULL;
 
   setlocale (LC_ALL, "");
 
-  gst_init_with_popt_table (&argc, &argv, options);
-  gst_tools_print_version ("gst-xmlinspect-0.8");
-  gst_control_init (&argc, &argv);
+  ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
+  g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
+  g_option_context_add_group (ctx, gst_init_get_option_group ());
+  if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
+    g_print ("Error initializing: %s\n", err->message);
+    exit (1);
+  }
+  g_option_context_free (ctx);
+
+  gst_tools_print_version ("gst-xmlinspect");
 
   PUT_STRING (0, "<?xml version=\"1.0\"?>");
 
@@ -904,34 +814,20 @@ main (int argc, char *argv[])
         GstPluginFeature *feature;
 
         /* FIXME implement other pretty print function for these */
-        feature = gst_registry_pool_find_feature (argv[1],
-            GST_TYPE_SCHEDULER_FACTORY);
-        if (feature) {
-          g_print ("%s: a scheduler\n", argv[1]);
-          return 0;
-        }
 #ifndef GST_DISABLE_INDEX
-        feature = gst_registry_pool_find_feature (argv[1],
+        feature = gst_default_registry_find_feature (argv[1],
             GST_TYPE_INDEX_FACTORY);
         if (feature) {
           g_print ("%s: an index\n", argv[1]);
           return 0;
         }
 #endif
-        feature = gst_registry_pool_find_feature (argv[1],
+        feature = gst_default_registry_find_feature (argv[1],
             GST_TYPE_TYPE_FIND_FACTORY);
         if (feature) {
           g_print ("%s: a type find function\n", argv[1]);
           return 0;
         }
-#ifndef GST_DISABLE_URI
-        feature = gst_registry_pool_find_feature (argv[1],
-            GST_TYPE_URI_HANDLER);
-        if (feature) {
-          g_print ("%s: an uri handler\n", argv[1]);
-          return 0;
-        }
-#endif
       }
     } else {
       /* strip the .so */
@@ -940,7 +836,7 @@ main (int argc, char *argv[])
     }
 
     /* otherwise assume it's a plugin */
-    plugin = gst_registry_pool_find_plugin (argv[1]);
+    plugin = gst_default_registry_find_plugin (argv[1]);
 
     /* if there is such a plugin, print out info */
 
